@@ -1,5 +1,5 @@
 /*
- * $Id: gfhost.c 10309 2016-11-30 23:32:30Z n-soda $
+ * $Id: gfhost.c 11267 2019-08-21 12:43:48Z n-soda $
  */
 
 #include <assert.h>
@@ -745,7 +745,8 @@ gfarm_paraccess_connect_request(void *closure)
 	}
 	e = gfs_client_connect_request_multiplexed(a->pa->q,
 	    a->canonical_hostname, a->port, gfm_client_username(gfm_server),
-	    &a->peer_addr, fs, gfarm_paraccess_connect_finish, a, &cs);
+	    &a->peer_addr, fs, gfarm_ctxp->schedule_rpc_timeout,
+	    gfarm_paraccess_connect_finish, a, &cs);
 	if (e != GFARM_ERR_NO_ERROR) {
 		gfarm_paraccess_callback(a->pa, a, &a->load, NULL, e);
 		return;
