@@ -1,5 +1,5 @@
 /*
- * $Id: gfarm_pfunc.c 10297 2016-11-24 14:32:48Z tatebe $
+ * $Id: gfarm_pfunc.c 11010 2019-02-07 14:40:33Z tatebe $
  */
 
 #include <stdio.h>
@@ -129,7 +129,8 @@ pfunc_check_disk_avail(
 	gfarm_error_t e;
 	gfarm_int32_t bsize;
 	gfarm_off_t blocks, bfree, bavail, files;
-	gfarm_off_t ffree, favail, avail;
+	gfarm_off_t ffree, favail;
+	gfarm_uint64_t avail;
 
 	e = gfs_statfsnode_by_path(
 		url, hostname, port, &bsize, &blocks, &bfree, &bavail,
@@ -137,8 +138,8 @@ pfunc_check_disk_avail(
 	if (e != GFARM_ERR_NO_ERROR)
 		return (e);
 	avail = bavail * bsize;
-	if (avail >= filesize &&
-	    avail >= gfarm_get_minimum_free_disk_space())
+	/* to reduce no space risk, keep minimum disk space */
+	if (avail >= filesize + gfarm_get_minimum_free_disk_space())
 		return (GFARM_ERR_NO_ERROR);
 	return (GFARM_ERR_NO_SPACE);
 }
