@@ -1,5 +1,5 @@
 /*
- * $Id: gfrep.c 7055 2012-11-09 14:53:49Z tatebe $
+ * $Id: gfrep.c 11101 2019-03-20 09:30:53Z tatebe $
  */
 
 #include <stdlib.h>
@@ -502,8 +502,6 @@ pfor(struct action *act, int nfinfo, struct file_info **finfo,
 		return ("no destination node");
 	if (nsrc <= 0)
 		return ("no source node");
-	if (ndst < opt_nrep)
-		return ("not enough number of destination nodes");
 	if (nfinfo <= 0)
 		return (NULL); /* no file */
 	/* sort 'sinfo' in descending order wrt file size */
