@@ -1,7 +1,7 @@
 /*
  * address constants which represents error conditions
  *
- * $Id: error.h 9714 2015-06-09 03:37:37Z tatebe $
+ * $Id: error.h 10309 2016-11-30 23:32:30Z n-soda $
  */
 
 /*
@@ -135,6 +135,8 @@ enum gfarm_errcode {
 		GFARM_ERR_CHECKSUM_MISMATCH,
 		GFARM_ERR_CONFLICT_DETECTED,
 		GFARM_ERR_INVALID_CREDENTIAL,
+		GFARM_ERR_NO_FILESYSTEM_NODE,
+		GFARM_ERR_DIRECTORY_QUOTA_EXISTS,
 
 		GFARM_ERR_NUMBER
 };
