@@ -1,5 +1,5 @@
 /*
- * $Id: gfs.h 10658 2017-11-02 13:02:27Z n-soda $
+ * $Id: gfs.h 11232 2019-06-28 17:17:30Z tatebe $
  */
 
 /*
@@ -122,7 +122,7 @@ gfarm_error_t gfs_pio_open(const char *, int, GFS_File *);
 gfarm_error_t gfs_pio_fhopen(gfarm_ino_t, gfarm_uint64_t, int, GFS_File *);
 gfarm_error_t gfs_pio_create(const char *, int, gfarm_mode_t, GFS_File *);
 gfarm_error_t gfs_pio_close(GFS_File);
-gfarm_error_t gfs_pio_close_getgen(GFS_File gf, gfarm_uint64_t *igenp);
+gfarm_error_t gfs_pio_close_getgen(GFS_File, gfarm_uint64_t *);
 
 int gfs_pio_eof(GFS_File);
 gfarm_error_t gfs_pio_error(GFS_File);
@@ -244,8 +244,20 @@ gfarm_error_t gfs_access(const char *, int);
 #define GFS_W_OK	2
 #define GFS_R_OK	4
 
-/* gfarm xattr prefix */
+/* gfarm xattr */
 #define GFARM_EA_PREFIX		"gfarm."
+#define GFARM_EA_PREFIX_LEN	6
+
+#define GFARM_EA_NCOPY_TYPE	"ncopy"
+#define GFARM_EA_NCOPY		GFARM_EA_PREFIX GFARM_EA_NCOPY_TYPE
+#define GFARM_EA_REPATTR_TYPE	"replicainfo"
+#define GFARM_EA_REPATTR	GFARM_EA_PREFIX GFARM_EA_REPATTR_TYPE
+
+/* Key name of virtual extended attribute for dirquota */
+#define GFARM_EA_DIRECTORY_QUOTA_TYPE	"directory_quota"
+#define GFARM_EA_DIRECTORY_QUOTA	\
+	GFARM_EA_PREFIX GFARM_EA_DIRECTORY_QUOTA_TYPE
+
 
 /* 5th argument (flags) of gfs_setxattr() and gfs_fsetxattr() */
 #define GFS_XATTR_CREATE    0x1     /* set value, fail if attr already exists */
@@ -433,9 +445,6 @@ gfarm_error_t gfs_acl_from_text_with_default(const char *, gfarm_acl_t *,
 gfarm_error_t gfs_acl_delete_mode(gfarm_acl_t);
 #endif
 
-/* Key name of virtual extended attribute for dirquota */
-#define GFARM_EA_DIRECTORY_QUOTA	GFARM_EA_PREFIX "directory_quota"
-
 /*
  * Key names of extended attribute for gfarm_root.*
  */
