@@ -21,6 +21,11 @@ enum gfarm_spool_check_level gfarm_spool_check_level_get(void);
 const char *gfarm_spool_check_level_get_by_name(void);
 gfarm_error_t gfarm_spool_check_level_set(enum gfarm_spool_check_level);
 gfarm_error_t gfarm_spool_check_level_set_by_name(const char *);
+#define GFARM_SPOOL_CHECK_PARALLEL_AUTOMATIC	0
+extern int gfarm_spool_check_parallel;
+extern int gfarm_spool_check_parallel_max;
+extern int gfarm_spool_check_parallel_step;
+extern gfarm_off_t gfarm_spool_check_parallel_per_capacity;
 extern float gfarm_spool_base_load;
 extern int gfarm_spool_digest_error_check;
 extern int gfarm_write_verify;
@@ -45,13 +50,26 @@ enum gfarm_backend_db_type {
 	GFARM_BACKEND_DB_TYPE_UNKNOWN,
 	GFARM_BACKEND_DB_TYPE_LDAP,
 	GFARM_BACKEND_DB_TYPE_POSTGRESQL,
-	GFARM_BACKEND_DB_TYPE_LOCALFS
+	GFARM_BACKEND_DB_TYPE_LOCALFS,
+	GFARM_BACKEND_DB_TYPE_NONE
 };
 extern enum gfarm_backend_db_type gfarm_backend_db_type;
 
-extern int gfarm_metadb_server_listen_backlog;
+enum gfarm_db_access_type {
+	GFARM_DB_ACCESS_TYPE_DBQ,
+	GFARM_DB_ACCESS_TYPE_JOURNAL,
+	GFARM_DB_ACCESS_TYPE_NONE
+};
+extern enum gfarm_db_access_type gfarm_db_access_type;
+
 extern int gfarm_xattr_size_limit;
 extern int gfarm_xmlattr_size_limit;
+extern int gfarm_directory_quota_count_per_user_limit;
+extern int gfarm_directory_quota_check_start_delay;
+extern int gfarm_directory_quota_check_retry_interval;
+extern int gfarm_quota_check_retry_interval;
+extern int gfarm_max_directory_depth;
+extern int gfarm_metadb_server_listen_backlog;
 extern int gfarm_metadb_version_major;
 extern int gfarm_metadb_version_minor;
 extern int gfarm_metadb_version_teeny;
@@ -59,25 +77,45 @@ extern int gfarm_metadb_max_descriptors;
 extern int gfarm_metadb_stack_size;
 extern int gfarm_metadb_thread_pool_size;
 extern int gfarm_metadb_job_queue_length;
+extern int gfarm_metadb_remover_queue_length;
+extern int gfarm_metadb_remove_scan_log_interval;
+extern int gfarm_metadb_remove_scan_interval_factor;
 extern int gfarm_metadb_heartbeat_interval;
 extern int gfarm_metadb_dbq_size;
 extern int gfarm_metadb_server_back_channel_sndbuf_limit;
 extern int gfarm_metadb_server_nfs_root_squash_support;
+
+enum gfarm_lock_type {
+	GFARM_LOCK_TYPE_MUTEX,
+	GFARM_LOCK_TYPE_TICKETLOCK,
+	GFARM_LOCK_TYPE_QUEUELOCK,
+	GFARM_LOCK_TYPE_LIMIT
+};
+extern int gfarm_metadb_server_long_term_lock_type;
+
 extern int gfarm_metadb_replica_remover_by_host_sleep_time;
 extern int gfarm_metadb_replica_remover_by_host_inode_step;
 extern int gfarm_replica_check;
 extern int gfarm_replica_check_remove;
+extern int gfarm_replica_check_remove_grace_used_space_ratio;
+extern int gfarm_replica_check_remove_grace_time;
 extern int gfarm_replica_check_reduced_log;
 extern int gfarm_replica_check_host_down_thresh;
 extern int gfarm_replica_check_sleep_time;
+extern int gfarm_replica_check_yield_time;
 extern int gfarm_replica_check_minimum_interval;
+extern int gfarm_replicainfo_enabled;
 #define GFARM_METADB_STACK_SIZE_DEFAULT 0 /* use OS default */
 #define GFARM_METADB_THREAD_POOL_SIZE_DEFAULT	16  /* quadcore, quadsocket */
 #if 0
 #define GFARM_METADB_JOB_QUEUE_LENGTH_DEFAULT	160 /* THREAD_POOL * 10 */
+#define GFARM_METADB_REMOVER_QUEUE_LENGTH_DEFAULT	16
 #else /* XXX FIXME: until bcworkq is implemented */
 #define GFARM_METADB_JOB_QUEUE_LENGTH_DEFAULT	16000
+#define GFARM_METADB_REMOVER_QUEUE_LENGTH_DEFAULT	160
 #endif
+#define GFARM_METADB_REMOVE_SCAN_LOG_INTERVAL_DEFAULT	3600 /* 3600 seconds */
+#define GFARM_METADB_REMOVE_SCAN_INTERVAL_FACTOR_DEFAULT 5 /* 1/5 */
 #define GFARM_METADB_HEARTBEAT_INTERVAL_DEFAULT 180 /* 3 min */
 #define GFARM_METADB_DBQ_SIZE_DEFAULT	65536
 #define GFARM_SYMLINK_LEVEL_MAX			20
@@ -115,6 +153,7 @@ extern int gfarm_iostat_max_client;
 /* miscellaneous configurations */
 extern char *gfarm_digest;
 extern int gfarm_simultaneous_replication_receivers;
+extern int gfarm_replication_busy_host;
 
 gfarm_error_t gfarm_get_global_username_by_host_for_connection_cache(
 	const char *, int, char **);
@@ -138,6 +177,7 @@ int gfarm_get_metadb_server_slave_replication_timeout(void);
 int gfarm_get_metadb_server_slave_max_size(void);
 int gfarm_get_metadb_server_force_slave(void);
 void gfarm_set_metadb_server_force_slave(int);
+int gfarm_is_db_access_type_none(void);
 
 /* configuration manipulation */
 
@@ -156,6 +196,10 @@ gfarm_error_t gfarm_config_copyout(const struct gfarm_config_type *,
 	union gfarm_config_storage *);
 
 gfarm_error_t gfarm_config_local_name_to_string(const char *, char *, size_t);
+gfarm_error_t gfarm_config_name_foreach(
+	gfarm_error_t (*)(void *, const char *), void *, int);
+#define GFARM_CONFIG_NAME_FLAG_FOR_METADB	1
+#define GFARM_CONFIG_NAME_FLAG_FOR_CLIENT	2
 
 struct gfm_connection;
 gfarm_error_t gfm_client_config_name_to_string(struct gfm_connection *,
