@@ -133,6 +133,7 @@ gfarm_context_init(void)
 	ctxp->attr_cache_limit = GFARM_CONFIG_MISC_DEFAULT;
 	ctxp->attr_cache_timeout = GFARM_CONFIG_MISC_DEFAULT;
 	ctxp->page_cache_timeout = GFARM_CONFIG_MISC_DEFAULT;
+	ctxp->schedule_rpc_timeout = GFARM_CONFIG_MISC_DEFAULT;
 	ctxp->schedule_cache_timeout = GFARM_CONFIG_MISC_DEFAULT;
 	ctxp->schedule_concurrency = GFARM_CONFIG_MISC_DEFAULT;
 	ctxp->schedule_concurrency_per_net = GFARM_CONFIG_MISC_DEFAULT;
@@ -145,6 +146,7 @@ gfarm_context_init(void)
 	ctxp->schedule_write_target_domain = NULL;
 	ctxp->schedule_write_local_priority = GFARM_CONFIG_MISC_DEFAULT;
 	ctxp->direct_local_access = GFARM_CONFIG_MISC_DEFAULT;
+	ctxp->replication_at_write_open = GFARM_CONFIG_MISC_DEFAULT;
 	ctxp->gfsd_connection_cache = GFARM_CONFIG_MISC_DEFAULT;
 	ctxp->gfmd_connection_cache = GFARM_CONFIG_MISC_DEFAULT;
 	ctxp->client_digest_check = GFARM_CONFIG_MISC_DEFAULT;
@@ -156,6 +158,7 @@ gfarm_context_init(void)
 	ctxp->ib_rdma = GFARM_CONFIG_MISC_DEFAULT;
 	ctxp->rdma_min_size = GFARM_CONFIG_MISC_DEFAULT;
 	ctxp->rdma_port = 0;
+	ctxp->rdma_gid_index = -1;
 	ctxp->rdma_device = NULL;
 	ctxp->rdma_mr_reg_mode = GFARM_CONFIG_MISC_DEFAULT;
 	ctxp->rdma_mr_reg_static_min_size = GFARM_CONFIG_MISC_DEFAULT;
