@@ -14,6 +14,7 @@ struct gfarm_context {
 	int attr_cache_limit;
 	int attr_cache_timeout;
 	int page_cache_timeout;
+	int schedule_rpc_timeout;
 	int schedule_cache_timeout;
 	int schedule_concurrency;
 	int schedule_concurrency_per_net;
@@ -26,6 +27,7 @@ struct gfarm_context {
 	char *schedule_write_target_domain;
 	int schedule_write_local_priority;
 	int direct_local_access;
+	int replication_at_write_open;
 	int gfmd_connection_cache;
 	int gfsd_connection_cache;
 	int client_digest_check;
@@ -40,6 +42,7 @@ struct gfarm_context {
 	int ib_rdma;
 	int rdma_min_size;
 	int rdma_port;
+	int rdma_gid_index;
 	char *rdma_device;
 	int rdma_mr_reg_mode;
 	int rdma_mr_reg_static_min_size;
