@@ -60,11 +60,12 @@
 #include "gfs_failover.h"
 #include "iostat.h"
 #ifdef __KERNEL__
+#undef HAVE_INFINIBAND
 #include "nanosec.h"
 #include "gfsk_fs.h"
 #endif /* __KERNEL__ */
 
-#define GFS_CLIENT_COMMAND_TIMEOUT	20 /* seconds */
+#include "gfs_rdma.h"
 
 #define XAUTH_NEXTRACT_MAXLEN	512
 
@@ -84,6 +85,10 @@ struct gfs_connection {
 	void *context; /* work area for RPC (esp. GFS_PROTO_COMMAND) */
 
 	int failover_count; /* compare to gfm_connection.failover_count */
+
+#ifdef HAVE_INFINIBAND
+	struct rdma_context *rdma_ctx; /* for client-gfsd rdma */
+#endif
 };
 
 #define staticp	(gfarm_ctxp->gfs_client_static)
@@ -101,6 +106,10 @@ struct gfs_client_static {
 #define SERVER_HASHTAB_SIZE	3079	/* prime number */
 
 static gfarm_error_t gfs_client_connection_dispose(void *);
+#ifdef HAVE_INFINIBAND
+static void gfs_ib_rdma_connect(struct gfs_connection *gfs_server);
+#endif
+static void gfs_ib_rdma_free(struct gfs_connection *gfs_server);
 
 gfarm_error_t
 gfs_client_static_init(struct gfarm_context *ctxp)
@@ -447,6 +456,15 @@ gfs_client_connection_alloc(const char *canonical_hostname,
 			gfarm_error_string(GFARM_ERR_NO_MEMORY));
 		return (GFARM_ERR_NO_MEMORY);
 	}
+
+#ifdef HAVE_INFINIBAND
+    /* RDMA */
+	if (!is_local)
+		gfs_rdma_init(0, &gfs_server->rdma_ctx);
+	else
+		gfs_server->rdma_ctx = NULL;
+#endif
+
 	gfs_server->port = ntohs(((struct sockaddr_in *)peer_addr)->sin_port);
 	gfs_server->is_local = is_local;
 	gfs_server->pid = 0;
@@ -488,7 +506,7 @@ gfs_client_connection_alloc_and_auth(const char *canonical_hostname,
 	if (e == GFARM_ERR_NO_ERROR)
 		e = gfarm_auth_request(gfs_server->conn, GFS_SERVICE_TAG,
 		    gfs_server->hostname, peer_addr, gfarm_get_auth_id_type(),
-		    user, &gfs_server->auth_method, NULL);
+		    user, NULL, &gfs_server->auth_method);
 	if (e == GFARM_ERR_NO_ERROR) {
 		/*
 		 * In GSI authentication, small packets are sent frequently,
@@ -506,10 +524,18 @@ gfs_client_connection_alloc_and_auth(const char *canonical_hostname,
 				    "is specified, but fails: %s",
 				    gfarm_error_string(e1));
 		}
+
+#ifdef HAVE_INFINIBAND
+		if (!gfs_server->is_local) {
+			/* not __KERNEL__, no need to lock */
+			gfs_ib_rdma_connect(gfs_server);
+		}
+#endif
 		*gfs_serverp = gfs_server;
 	} else {
 		free(gfs_server->hostname);
 		gfp_xdr_free(gfs_server->conn);
+		gfs_ib_rdma_free(gfs_server);
 		free(gfs_server);
 		gflog_debug(GFARM_MSG_1001183,
 			"connection or authentication failed: %s",
@@ -610,6 +636,7 @@ gfs_client_connection_dispose(void *connection_data)
 
 	gfp_uncached_connection_dispose(gfs_server->cache_entry);
 	free(gfs_server->hostname);
+	gfs_ib_rdma_free(gfs_server);
 	/* XXX - gfs_server->context should be NULL here */
 	free(gfs_server);
 	return (e);
@@ -989,6 +1016,7 @@ struct gfs_client_connect_state {
 	struct gfarm_eventqueue *q;
 	struct gfarm_event *writable;
 	struct sockaddr peer_addr;
+	int connect_timeout;
 	void (*continuation)(void *);
 	void *closure;
 
@@ -1051,9 +1079,10 @@ gfs_client_connect_start_auth(int events, int fd, void *closure,
 		    state->gfs_server->hostname,
 		    &state->peer_addr,
 		    gfarm_get_auth_id_type(),
-		    gfs_client_username(state->gfs_server),
+		    gfs_client_username(state->gfs_server), NULL,
+		    state->connect_timeout,
 		    gfs_client_connect_end_auth, state,
-		    &state->auth_state, NULL);
+		    &state->auth_state);
 		if (state->error == GFARM_ERR_NO_ERROR) {
 			/*
 			 * call auth_request,
@@ -1101,6 +1130,7 @@ gfarm_error_t
 gfs_client_connect_request_multiplexed(struct gfarm_eventqueue *q,
 	const char *canonical_hostname, int port, const char *user,
 	struct sockaddr *peer_addr, struct gfarm_filesystem *fs,
+	int connect_timeout,
 	void (*continuation)(void *), void *closure,
 	struct gfs_client_connect_state **statepp)
 {
@@ -1154,6 +1184,7 @@ gfs_client_connect_request_multiplexed(struct gfarm_eventqueue *q,
 
 	state->q = q;
 	state->peer_addr = *peer_addr;
+	state->connect_timeout = connect_timeout;
 	state->continuation = continuation;
 	state->closure = closure;
 	state->gfs_server = gfs_server;
@@ -1180,9 +1211,9 @@ gfs_client_connect_request_multiplexed(struct gfarm_eventqueue *q,
 		e = gfarm_auth_request_multiplexed(q,
 		    gfs_server->conn, GFS_SERVICE_TAG,
 		    gfs_server->hostname, &state->peer_addr,
-		    gfarm_get_auth_id_type(), user,
+		    gfarm_get_auth_id_type(), user, NULL, connect_timeout,
 		    gfs_client_connect_end_auth, state,
-		    &state->auth_state, NULL);
+		    &state->auth_state);
 		if (e == GFARM_ERR_NO_ERROR) {
 			*statepp = state;
 			/*
@@ -1459,7 +1490,7 @@ gfs_client_vrpc(struct gfs_connection *gfs_server, int just, int do_timeout,
 		 * We just use gfarm_error_t as the errcode,
 		 * Note that GFARM_ERR_NO_ERROR == 0.
 		 */
-		gflog_debug(GFARM_MSG_1003562, "gfp_xdr_vrpc(%d) errcode=%d",
+		gflog_debug(GFARM_MSG_1003562, "gfp_xdr_vrpc(%d): errcode=%d",
 		    command, errcode);
 		return (errcode);
 	}
@@ -1837,6 +1868,7 @@ struct gfs_client_statfs_state {
 	void *closure;
 	struct gfs_connection *gfs_server;
 	char *path;
+	int statfs_timeout;
 
 	/* results */
 	gfarm_error_t error;
@@ -1863,7 +1895,7 @@ gfs_client_statfs_send_request(int events, int fd, void *closure,
 			gfs_client_purge_from_cache(state->gfs_server);
 		}
 		if (state->error == GFARM_ERR_NO_ERROR) {
-			timeout.tv_sec = GFS_CLIENT_COMMAND_TIMEOUT;
+			timeout.tv_sec = state->statfs_timeout;
 			timeout.tv_usec = 0;
 			if ((rv = gfarm_eventqueue_add_event(state->q,
 			    state->readable, &timeout)) == 0) {
@@ -1885,10 +1917,21 @@ gfs_client_statfs_recv_result(int events, int fd, void *closure,
 	const struct timeval *t)
 {
 	struct gfs_client_statfs_state *state = closure;
+	gfarm_error_t e;
 
 	if ((events & GFARM_EVENT_TIMEOUT) != 0) {
 		assert(events == GFARM_EVENT_TIMEOUT);
 		state->error = GFARM_ERR_OPERATION_TIMED_OUT;
+		gfs_client_execute_hook_for_connection_error(
+		    state->gfs_server);
+		gfs_client_purge_from_cache(state->gfs_server);
+		e = gfp_xdr_shutdown(state->gfs_server->conn);
+		if (e != GFARM_ERR_NO_ERROR) {
+			gflog_debug(GFARM_MSG_1005093,
+			    "gfs_client_statfs_recv_result: shutdown: %s",
+			    gfarm_error_string(e));
+		}
+
 	} else {
 		assert(events == GFARM_EVENT_READ);
 		state->error = gfs_client_rpc_result(state->gfs_server, 0,
@@ -1902,7 +1945,7 @@ gfs_client_statfs_recv_result(int events, int fd, void *closure,
 
 gfarm_error_t
 gfs_client_statfs_request_multiplexed(struct gfarm_eventqueue *q,
-	struct gfs_connection *gfs_server, char *path,
+	struct gfs_connection *gfs_server, char *path, int statfs_timeout,
 	void (*continuation)(void *), void *closure,
 	struct gfs_client_statfs_state **statepp)
 {
@@ -1924,6 +1967,7 @@ gfs_client_statfs_request_multiplexed(struct gfarm_eventqueue *q,
 	state->closure = closure;
 	state->gfs_server = gfs_server;
 	state->path = path;
+	state->statfs_timeout = statfs_timeout;
 	state->error = GFARM_ERR_NO_ERROR;
 	state->writable = gfarm_fd_event_alloc(GFARM_EVENT_WRITE,
 	    gfs_client_connection_fd(gfs_server),
@@ -1996,6 +2040,265 @@ gfs_client_statfs_result_multiplexed(struct gfs_client_statfs_state *state,
 	return (e);
 }
 
+/*--------------------------------------------------------*/
+/*
+ * multiplexed version of gfs_ib_rdma()
+ */
+struct gfs_ib_rdma_state {
+	struct gfarm_eventqueue *q;
+	struct gfarm_event *writable, *readable;
+	void (*continuation)(void *);
+	void *closure;
+	struct gfs_connection *gfs_server;
+	int ib_rdma_timeout;
+
+	/* results */
+	gfarm_error_t error;
+	enum rdma_state {RDMA_EXCH_INFO, RDMA_HELLO, RDMA_FIN} state;
+};
+
+static void
+gfs_ib_rdma_send_request(int events, int fd, void *closure,
+	const struct timeval *t)
+{
+	struct gfs_ib_rdma_state *state = closure;
+	int rv;
+	struct timeval timeout;
+	struct rdma_context *ctx = gfs_ib_rdma_context(state->gfs_server);
+
+	switch (state->state) {
+	case RDMA_EXCH_INFO:
+	{
+		gfarm_uint32_t local_lid = gfs_rdma_get_local_lid(ctx);
+		gfarm_uint32_t local_qpn = gfs_rdma_get_local_qpn(ctx);
+		gfarm_uint32_t local_psn = gfs_rdma_get_local_psn(ctx);
+		void *local_gid = gfs_rdma_get_local_gid(ctx);
+		size_t  size = gfs_rdma_get_gid_size();
+
+		state->error = gfs_client_rpc_request(state->gfs_server,
+			GFS_PROTO_RDMA_EXCH_INFO, "iiib", local_lid, local_qpn,
+			local_psn, size, local_gid);
+		break;
+	}
+	case RDMA_HELLO:
+	{
+		gfarm_uint32_t rkey = gfs_rdma_get_rkey(gfs_rdma_get_mr(ctx));
+		unsigned char *buf = gfs_rdma_get_buffer(ctx);
+		int size = gfs_rdma_get_gid_size();
+		gfarm_uint64_t addr;
+		memcpy(buf, gfs_rdma_get_local_gid(ctx), size);
+		addr = (uintptr_t)buf;
+		state->error = gfs_client_rpc_request(state->gfs_server,
+			GFS_PROTO_RDMA_HELLO, "iil", rkey, size, addr);
+		break;
+	}
+	default:
+		gflog_fatal(GFARM_MSG_1004537,
+			"request state error: %d", state->state);
+	}
+	gflog_debug(GFARM_MSG_1004538,
+		"gfs_ib_rdma_send_request:%s state=%d, %s",
+		state->gfs_server->hostname, state->state,
+		gfarm_error_string(state->error));
+	if (state->error == GFARM_ERR_NO_ERROR) {
+		state->error = gfp_xdr_flush(state->gfs_server->conn);
+		if (IS_CONNECTION_ERROR(state->error)) {
+			gfs_client_execute_hook_for_connection_error(
+			    state->gfs_server);
+			gfs_client_purge_from_cache(state->gfs_server);
+		} else if (state->error == GFARM_ERR_NO_ERROR) {
+			timeout.tv_sec = state->ib_rdma_timeout;
+			timeout.tv_usec = 0;
+			if ((rv = gfarm_eventqueue_add_event(state->q,
+			    state->readable, &timeout)) == 0) {
+				/* go to gfs_client_statfs_recv_result() */
+				return;
+			}
+			state->error = gfarm_errno_to_error(rv);
+		}
+	}
+	gflog_debug(GFARM_MSG_1004539,
+		"request for client rdma failed: %s",
+		gfarm_error_string(state->error));
+	if (state->continuation != NULL)
+		(*state->continuation)(state->closure);
+}
+
+static void
+gfs_ib_rdma_recv_result(int events, int fd, void *closure,
+	const struct timeval *t)
+{
+	gfarm_error_t e;
+	struct gfs_ib_rdma_state *state = closure;
+	struct rdma_context *ctx = gfs_ib_rdma_context(state->gfs_server);
+
+	if ((events & GFARM_EVENT_TIMEOUT) != 0) {
+		assert(events == GFARM_EVENT_TIMEOUT);
+		state->error = GFARM_ERR_OPERATION_TIMED_OUT;
+		gfs_client_execute_hook_for_connection_error(
+		    state->gfs_server);
+		gfs_client_purge_from_cache(state->gfs_server);
+		e = gfp_xdr_shutdown(state->gfs_server->conn);
+		if (e != GFARM_ERR_NO_ERROR) {
+			gflog_debug(GFARM_MSG_1005094,
+			    "gfs_ib_rdma_recv_result: shutdown: %s",
+			    gfarm_error_string(e));
+		}
+	} else {
+		assert(events == GFARM_EVENT_READ);
+		switch (state->state) {
+		case RDMA_EXCH_INFO:
+		{
+			gfarm_uint32_t server_lid;
+			gfarm_uint32_t server_qpn;
+			gfarm_uint32_t server_psn;
+			char server_gid[IBV_GID_SIZE];
+			size_t  size = gfs_rdma_get_gid_size();
+			int success;
+
+			state->error = gfs_client_rpc_result(state->gfs_server,
+				0, "iiiib", &success, &server_lid, &server_qpn,
+				&server_psn, size, &size, server_gid);
+			if (state->error == GFARM_ERR_NO_ERROR && success) {
+				gfs_rdma_set_remote_lid(ctx, server_lid);
+				gfs_rdma_set_remote_qpn(ctx, server_qpn);
+				gfs_rdma_set_remote_psn(ctx, server_psn);
+				gfs_rdma_set_remote_gid(ctx, server_gid);
+
+				state->error = gfs_rdma_connect(ctx);
+
+			} else if (state->error == GFARM_ERR_NO_ERROR)
+				state->error = GFARM_ERR_DEVICE_NOT_CONFIGURED;
+			break;
+		}
+		case RDMA_HELLO:
+			state->error = gfs_client_rpc_result(state->gfs_server,
+				0, "");
+			break;
+		default:
+			gflog_fatal(GFARM_MSG_1004540,
+			"result state error: %d", state->state);
+		}
+	}
+	gflog_debug(GFARM_MSG_1004541,
+		"gfs_ib_rdma_recv_result:%s state=%d, %s",
+		state->gfs_server->hostname, state->state,
+		gfarm_error_string(state->error));
+	if (state->error == GFARM_ERR_NO_ERROR) {
+		state->state++;
+		if (state->state != RDMA_FIN) {
+			int rv;
+			if ((rv = gfarm_eventqueue_add_event(state->q,
+			    state->writable, NULL)) == 0) {
+				/* go to gfs_ib_rdma_recv_result() */
+				return;
+			}
+			state->error = gfarm_errno_to_error(rv);
+		}
+
+	}
+	if (state->error != GFARM_ERR_NO_ERROR) {
+		gfs_rdma_disable(ctx);
+	}
+	if (state->continuation != NULL)
+		(*state->continuation)(state->closure);
+}
+
+gfarm_error_t
+gfs_ib_rdma_request_multiplexed(struct gfarm_eventqueue *q,
+	struct gfs_connection *gfs_server, int ib_rdma_timeout,
+	void (*continuation)(void *), void *closure,
+	struct gfs_ib_rdma_state **statepp)
+{
+	gfarm_error_t e;
+	int rv;
+	struct gfs_ib_rdma_state *state;
+	struct rdma_context *ctx = gfs_ib_rdma_context(gfs_server);
+
+	if (!gfs_rdma_check(ctx)) {
+		return (GFARM_ERR_DEVICE_NOT_CONFIGURED);
+	}
+
+	GFARM_MALLOC(state);
+	if (state == NULL) {
+		e = GFARM_ERR_NO_MEMORY;
+		gflog_debug(GFARM_MSG_1004542,
+			"allocation of client rdma state failed: %s",
+			gfarm_error_string(GFARM_ERR_NO_MEMORY));
+		goto error_return;
+	}
+
+	state->q = q;
+	state->continuation = continuation;
+	state->closure = closure;
+	state->gfs_server = gfs_server;
+	state->ib_rdma_timeout = ib_rdma_timeout;
+	state->error = GFARM_ERR_NO_ERROR;
+	state->state = RDMA_EXCH_INFO;
+	state->writable = gfarm_fd_event_alloc(GFARM_EVENT_WRITE,
+	    gfs_client_connection_fd(gfs_server),
+	    gfs_ib_rdma_send_request, state);
+	if (state->writable == NULL) {
+		e = GFARM_ERR_NO_MEMORY;
+		gflog_debug(GFARM_MSG_1004543,
+			"allocation of state->writable failed: %s",
+			gfarm_error_string(GFARM_ERR_NO_MEMORY));
+		goto error_free_state;
+	}
+	/*
+	 * We cannot use two independent events (i.e. a fd_event with
+	 * GFARM_EVENT_READ flag and a timer_event) here, because
+	 * it's possible that both event handlers are called at once.
+	 */
+	state->readable = gfarm_fd_event_alloc(
+	    GFARM_EVENT_READ|GFARM_EVENT_TIMEOUT,
+	    gfs_client_connection_fd(gfs_server),
+	    gfs_ib_rdma_recv_result, state);
+	if (state->readable == NULL) {
+		e = GFARM_ERR_NO_MEMORY;
+		gflog_debug(GFARM_MSG_1004544,
+			"allocation of state->readable failed: %s",
+			gfarm_error_string(GFARM_ERR_NO_MEMORY));
+		goto error_free_writable;
+	}
+	/* go to gfs_ib_rdma_send_request() */
+	rv = gfarm_eventqueue_add_event(q, state->writable, NULL);
+	if (rv != 0) {
+		e = gfarm_errno_to_error(rv);
+		gflog_debug(GFARM_MSG_1004545,
+			"adding event to event queue failed: %s",
+			gfarm_error_string(e));
+		goto error_free_readable;
+	}
+	*statepp = state;
+	return (GFARM_ERR_NO_ERROR);
+
+error_free_readable:
+	gfarm_event_free(state->readable);
+error_free_writable:
+	gfarm_event_free(state->writable);
+error_free_state:
+	free(state);
+error_return:
+	return (e);
+}
+
+gfarm_error_t
+gfs_ib_rdma_result_multiplexed(struct gfs_ib_rdma_state *state)
+{
+	gfarm_error_t e = state->error;
+
+	gfarm_event_free(state->writable);
+	gfarm_event_free(state->readable);
+	free(state);
+	return (e);
+}
+/*---------------------------------------------*/
+#ifndef __KERNEL__
+#define GFS_STACK_BUFSIZE  GFS_PROTO_MAX_IOSIZE
+#else /* __KERNEL__  */
+#define GFS_STACK_BUFSIZE 1024
+#endif /* __KERNEL__  */
 /*
  * commonly used by both clients and gfsd
  *
@@ -2020,7 +2323,7 @@ gfs_sendfile_common(struct gfp_xdr *conn, gfarm_int32_t *src_errp,
 	ssize_t rv;
 	off_t sent = 0;
 	int mode_unknown = 1, mode_thread_safe = 1, until_eof = len < 0;
-	char buffer[GFS_PROTO_MAX_IOSIZE];
+	char buffer[GFS_STACK_BUFSIZE];
 #if 0 /* not yet in gfarm v2 */
 	struct gfs_client_rep_rate_info *rinfo = NULL;
 
@@ -2032,9 +2335,9 @@ gfs_sendfile_common(struct gfp_xdr *conn, gfarm_int32_t *src_errp,
 #endif
 	if (until_eof || len > 0) {
 		for (;;) {
-			to_read = until_eof ? GFS_PROTO_MAX_IOSIZE :
-			    len < GFS_PROTO_MAX_IOSIZE ?
-			    len : GFS_PROTO_MAX_IOSIZE;
+			to_read = until_eof ? GFS_STACK_BUFSIZE :
+			    len < GFS_STACK_BUFSIZE ?
+			    len : GFS_STACK_BUFSIZE;
 			if (mode_unknown) {
 				mode_unknown = 0;
 				rv = pread(r_fd, buffer, to_read, r_off);
@@ -2139,7 +2442,7 @@ gfs_recvfile_common(struct gfp_xdr *conn, gfarm_int32_t *dst_errp,
 		do {
 			int i, partial;
 			ssize_t rv;
-			char buffer[GFS_PROTO_MAX_IOSIZE];
+			char buffer[GFS_STACK_BUFSIZE];
 
 			/* XXX - FIXME layering violation */
 			e = gfp_xdr_recv_partial(conn, 0,
@@ -2164,7 +2467,8 @@ gfs_recvfile_common(struct gfp_xdr *conn, gfarm_int32_t *dst_errp,
 					mode_unknown = 0;
 					rv = pwrite(w_fd,
 					    buffer + i, partial - i, w_off);
-					if (rv == -1 && errno == ESPIPE &&
+					if (rv == -1 && (errno == ESPIPE ||
+						errno == ENXIO) &&
 					    w_off <= 0) {
 						mode_thread_safe = 0;
 						rv = write(w_fd,
@@ -2302,7 +2606,6 @@ gfs_client_recvfile(struct gfs_connection *gfs_server,
 			    &src_err);
 			if (e2 == GFARM_ERR_NO_ERROR && eof)
 				e2 = GFARM_ERR_PROTOCOL;
-			
 		}
 		if (IS_CONNECTION_ERROR(e) || IS_CONNECTION_ERROR(e2)) {
 			gfs_client_execute_hook_for_connection_error(
@@ -2686,3 +2989,176 @@ gfs_client_get_load_result_multiplexed(
 	free(state);
 	return (error);
 }
+
+#ifdef HAVE_INFINIBAND
+gfarm_error_t
+gfs_ib_rdma_exch_info(struct gfs_connection *gfs_server)
+{
+	gfarm_error_t e;
+	struct rdma_context *ctx = gfs_ib_rdma_context(gfs_server);
+	int success;
+	gfarm_uint32_t server_lid;
+	gfarm_uint32_t server_qpn;
+	gfarm_uint32_t server_psn;
+	char server_gid[IBV_GID_SIZE];
+	gfarm_uint32_t local_lid = gfs_rdma_get_local_lid(ctx);
+	gfarm_uint32_t local_qpn = gfs_rdma_get_local_qpn(ctx);
+	gfarm_uint32_t local_psn = gfs_rdma_get_local_psn(ctx);
+	void	*local_gid = gfs_rdma_get_local_gid(ctx);
+	size_t	size = gfs_rdma_get_gid_size();
+
+	e = gfs_client_rpc(gfs_server, 0, GFS_PROTO_RDMA_EXCH_INFO,
+		"iiib/iiiib",
+		local_lid, local_qpn, local_psn, size, local_gid,
+		&success, &server_lid, &server_qpn, &server_psn,
+		size, &size, server_gid);
+
+	if (e == GFARM_ERR_NO_ERROR && success) {
+		gfs_rdma_set_remote_lid(ctx, server_lid);
+		gfs_rdma_set_remote_qpn(ctx, server_qpn);
+		gfs_rdma_set_remote_psn(ctx, server_psn);
+		gfs_rdma_set_remote_gid(ctx, server_gid);
+
+		gfs_rdma_enable(ctx);
+
+		gflog_debug(GFARM_MSG_1004546,
+			"gfs_ib_rdma_exch_info:%s success",
+				gfs_server->hostname);
+	} else {
+		gflog_debug(GFARM_MSG_1004547,
+			"gfs_ib_rdma_exch_info:%s failed, %s",
+				gfs_server->hostname, gfarm_error_string(e));
+		if (e == GFARM_ERR_NO_ERROR)
+			e = GFARM_ERR_DEVICE_NOT_CONFIGURED;
+
+		gfs_rdma_disable(ctx);
+	}
+
+	return (e);
+}
+gfarm_error_t
+gfs_ib_rdma_hello(struct gfs_connection *gfs_server)
+{
+	gfarm_error_t e;
+	struct rdma_context *ctx = gfs_ib_rdma_context(gfs_server);
+	gfarm_uint32_t rkey = gfs_rdma_get_rkey(gfs_rdma_get_mr(ctx));
+	unsigned char *buf;
+	gfarm_uint64_t addr;
+	int size;
+
+	buf = gfs_rdma_get_buffer(ctx);
+	size = gfs_rdma_get_gid_size();
+	memcpy(buf, gfs_rdma_get_local_gid(ctx), size);
+	addr = (uintptr_t)buf;
+
+	e = gfs_client_rpc(gfs_server, 0, GFS_PROTO_RDMA_HELLO, "iil/",
+				rkey, size, addr);
+	if (e == GFARM_ERR_NO_ERROR) {
+		gfs_rdma_enable(ctx);
+		gflog_debug(GFARM_MSG_1004548,
+				"gfs_ib_rdma_hello: success");
+	} else {
+		gflog_debug(GFARM_MSG_1004549,
+			"gfs_ib_rdma_hello: failed, %s",
+			gfarm_error_string(e));
+		gfs_rdma_disable(ctx);
+	}
+	return (e);
+}
+
+gfarm_error_t
+gfs_ib_rdma_pread(struct gfs_connection *gfs_server,
+		gfarm_int32_t fd, void *buffer, size_t size,
+		gfarm_off_t off, size_t *np, gfarm_uint32_t rkey)
+{
+	gfarm_error_t e;
+	gfarm_uint64_t addr = (uintptr_t)buffer;
+	gfarm_uint32_t n;
+
+	if ((e = gfs_client_rpc(gfs_server, 0, GFS_PROTO_RDMA_PREAD, "iilil/i",
+		fd, (int)size, off, rkey, addr, &n)) != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1004550,
+					"gfs_client_rpc() failed: %s",
+					gfarm_error_string(e));
+		return (e);
+	}
+
+	*np = n;
+
+	if (*np > size) {
+		gflog_debug(GFARM_MSG_1004551,
+			"Protocol error in client rdma_pread (%llu)>(%llu)",
+			(unsigned long long)*np, (unsigned long long)size);
+		return (GFARM_ERRMSG_GFS_PROTO_PREAD_PROTOCOL);
+	}
+
+	return (GFARM_ERR_NO_ERROR);
+}
+
+gfarm_error_t
+gfs_ib_rdma_pwrite(struct gfs_connection *gfs_server,
+			gfarm_int32_t fd, const void *buffer, size_t size,
+			gfarm_off_t off, size_t *np, gfarm_uint32_t rkey)
+{
+	gfarm_error_t e;
+	gfarm_uint64_t addr = (uintptr_t)buffer;
+	gfarm_uint32_t n;
+
+	if ((e = gfs_client_rpc(gfs_server, 0, GFS_PROTO_RDMA_PWRITE, "iilil/i",
+		fd, (int)size, off, rkey, addr, &n)) != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1004552, "gfs_client_rpc() failed: %s",
+				gfarm_error_string(e));
+		return (e);
+	}
+
+	*np = n;
+
+	if (*np > size) {
+		gflog_debug(GFARM_MSG_1004553,
+			"Protocol error in client rdma_pwrite (%llu)>(%llu)",
+			(unsigned long long)*np, (unsigned long long)size);
+		return (GFARM_ERRMSG_GFS_PROTO_PWRITE_PROTOCOL);
+	}
+
+	return (GFARM_ERR_NO_ERROR);
+}
+
+struct rdma_context *
+gfs_ib_rdma_context(struct gfs_connection *gfs_server)
+{
+	return (gfs_server->rdma_ctx);
+}
+
+static void
+gfs_ib_rdma_free(struct gfs_connection *gfs_server)
+{
+	if (gfs_server->rdma_ctx) {
+		gfs_rdma_finish(gfs_server->rdma_ctx);
+		gfs_server->rdma_ctx = NULL;
+	}
+}
+
+static void
+gfs_ib_rdma_connect(struct gfs_connection *gfs_server)
+{
+	struct rdma_context *ctx = gfs_ib_rdma_context(gfs_server);
+
+	if (!gfs_rdma_check(ctx))
+		return;
+	if (gfs_ib_rdma_exch_info(gfs_server) != GFARM_ERR_NO_ERROR)
+		return;
+	if (gfs_rdma_connect(ctx) != GFARM_ERR_NO_ERROR)
+		return;
+	gfs_ib_rdma_hello(gfs_server);
+}
+#else
+struct rdma_context *
+gfs_ib_rdma_context(struct gfs_connection *gfs_server)
+{
+	return (NULL);
+}
+static void
+gfs_ib_rdma_free(struct gfs_connection *gfs_server)
+{
+}
+#endif
