@@ -1,5 +1,5 @@
 /*
- * $Id: gfs_client.h 10264 2016-11-10 04:47:25Z gfarm-suri $
+ * $Id: gfs_client.h 11267 2019-08-21 12:43:48Z n-soda $
  */
 
 #define GFS_CONN_RETRY_COUNT	1	/* retry for gfsd connection error */
@@ -51,7 +51,7 @@ gfarm_error_t gfs_client_connection_enter_cache_tail(struct gfs_connection *);
 struct gfarm_filesystem;
 gfarm_error_t gfs_client_connect_request_multiplexed(
 	struct gfarm_eventqueue *, const char *, int, const char *,
-	struct sockaddr *, struct gfarm_filesystem *fs,
+	struct sockaddr *, struct gfarm_filesystem *fs, int,
 	void (*)(void *), void *, struct gfs_client_connect_state **);
 gfarm_error_t gfs_client_connect_result_multiplexed(
 	struct gfs_client_connect_state *,
@@ -118,7 +118,7 @@ gfarm_error_t gfs_client_statfs(struct gfs_connection *, char *,
 
 struct gfs_client_statfs_state;
 gfarm_error_t gfs_client_statfs_request_multiplexed(struct gfarm_eventqueue *,
-	struct gfs_connection *, char *, void (*)(void *), void *,
+	struct gfs_connection *, char *, int, void (*)(void *), void *,
 	struct gfs_client_statfs_state **);
 gfarm_error_t gfs_client_statfs_result_multiplexed(
 	struct gfs_client_statfs_state *,
@@ -193,7 +193,7 @@ void gfs_client_connection_unlock(struct gfs_connection *gfs_server);
 
 struct gfs_ib_rdma_state;
 gfarm_error_t gfs_ib_rdma_request_multiplexed(struct gfarm_eventqueue *q,
-	struct gfs_connection *gfs_server, void (*continuation)(void *),
+	struct gfs_connection *gfs_server, int, void (*continuation)(void *),
 	void *closure, struct gfs_ib_rdma_state **statepp);
 gfarm_error_t gfs_ib_rdma_result_multiplexed(struct gfs_ib_rdma_state *state);
 
