@@ -1,5 +1,5 @@
 /*
- * $Id: gfs_pio.c 10737 2017-12-06 04:34:43Z tatebe $
+ * $Id: gfs_pio.c 11107 2019-03-20 14:26:12Z tatebe $
  */
 
 #include <sys/types.h> /* mode_t */
@@ -25,6 +25,7 @@
 #include "queue.h"
 #include "thrsubr.h"
 
+#include "config.h"
 #include "context.h"
 #include "liberror.h"
 #include "filesystem.h"
@@ -438,6 +439,132 @@ gfs_file_free(GFS_File gf)
 	free(gf);
 }
 
+static gfarm_error_t
+count_incomplete(const char *url, int *np)
+{
+	gfarm_error_t e;
+	struct gfs_replica_info *ri;
+	int i, n, num_incomplete = 0;
+
+	e = gfs_replica_info_by_name(url,
+	    GFS_REPLICA_INFO_INCLUDING_INCOMPLETE_COPY, &ri);
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1005073,
+		    "gfs_replica_info_by_name(%s): %s",
+		    url, gfarm_error_string(e));
+		return (e);
+	}
+	n = gfs_replica_info_number(ri);
+	for (i = 0; i < n; i++)
+		if (gfs_replica_info_nth_is_incomplete(ri, i))
+			++num_incomplete;
+	gfs_replica_info_free(ri);
+	*np = num_incomplete;
+	return (e);
+}
+
+static gfarm_error_t
+wait_for_replication(const char *url)
+{
+	gfarm_error_t e;
+	int n;
+
+	while ((e = count_incomplete(url, &n)) == GFARM_ERR_NO_ERROR) {
+		if (n == 0)
+			break;
+		gflog_info(GFARM_MSG_1005074,
+		    "%s: wait for %d replica creation%s, "
+		    "sleep 1 sec", url, n, n > 1 ? "s" : "");
+		sleep(1);
+	}
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1005075,
+		    "wait_for_replication(%s), count_incomplete: %s",
+		    url, gfarm_error_string(e));
+	}
+	return (e);
+}
+
+static gfarm_error_t
+replicate_to_writable_host(const char *url)
+{
+	gfarm_error_t e;
+	int i, available_nhosts, nhosts, *ports;
+	struct gfarm_host_sched_info *available_hosts;
+	char **hosts;
+
+	e = gfarm_schedule_hosts_domain_all(url, "", &available_nhosts,
+	    &available_hosts);
+	if (e != GFARM_ERR_NO_ERROR)
+		return (e);
+
+	nhosts = available_nhosts;
+	GFARM_MALLOC_ARRAY(hosts, nhosts);
+	GFARM_MALLOC_ARRAY(ports, nhosts);
+	if (hosts == NULL || ports == NULL) {
+		e = GFARM_ERR_NO_MEMORY;
+		gflog_debug(GFARM_MSG_1005076,
+		    "allocation of hosts and ports failed: %s",
+		    gfarm_error_string(e));
+		goto finish;
+	}
+	e = gfarm_schedule_hosts_acyclic_to_write(url,
+	    available_nhosts, available_hosts, &nhosts, hosts, ports);
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1005077,
+		    "gfarm_schedule_hosts_acyclic_to_write: %s",
+		    gfarm_error_string(e));
+	} else {
+		for (i = 0; i < nhosts; i++) {
+			/* UNCONST */
+			e = gfs_replicate_to((char *)url, hosts[i], ports[i]);
+			if (e == GFARM_ERR_NO_ERROR ||
+			    e == GFARM_ERR_OPERATION_ALREADY_IN_PROGRESS)
+				break;
+			else
+				gflog_debug(GFARM_MSG_1005078,
+				    "gfs_replicate_to: %s",
+				    gfarm_error_string(e));
+		}
+	}
+finish:
+	gfarm_host_sched_info_free(available_nhosts, available_hosts);
+	free(hosts);
+	free(ports);
+	return (e);
+}
+
+static gfarm_error_t
+open_common(int create,
+	const char *path, int flags, gfarm_mode_t mode,
+	struct gfm_connection **gfm_serverp, int *fdp, int *typep,
+	gfarm_ino_t *inump, gfarm_uint64_t *igenp, char **urlp,
+	struct gfs_pio_internal_cksum_info *cksum_info)
+{
+	gfarm_error_t e;
+
+retry:
+	if (create)
+		e = gfm_create_fd(path, flags, mode, gfm_serverp, fdp, typep,
+		    inump, igenp, urlp, cksum_info);
+	else
+		e = gfm_open_fd(path, flags, gfm_serverp, fdp, typep,
+		    urlp, inump, igenp, cksum_info);
+	if (e == GFARM_ERR_NO_SPACE && gfarm_ctxp->replication_at_write_open) {
+		gfarm_error_t e2 = replicate_to_writable_host(path);
+
+		if (e2 == GFARM_ERR_NO_ERROR)
+			goto retry;
+		else if (e2 == GFARM_ERR_OPERATION_ALREADY_IN_PROGRESS) {
+			e2 = wait_for_replication(path);
+			if (e2 == GFARM_ERR_NO_ERROR)
+				goto retry;
+		}
+		/* GFARM_ERR_NO_SPACE when e2 is error */
+	}
+	return (e);
+}
+
 gfarm_error_t
 gfs_pio_create_igen(const char *url, int flags, gfarm_mode_t mode,
 	GFS_File *gfp, gfarm_ino_t *inop, gfarm_uint64_t *genp)
@@ -458,8 +585,9 @@ gfs_pio_create_igen(const char *url, int flags, gfarm_mode_t mode,
 	GFARM_TIMEVAL_FIX_INITIALIZE_WARNING(t1);
 	gfs_profile(gfarm_gettimerval(&t1));
 
-	if ((e = gfm_create_fd(url, flags, mode, &gfm_server, &fd, &type,
-	    &inum, &gen, &real_url, cip)) == GFARM_ERR_NO_ERROR) {
+	e = open_common(1, url, flags, mode, &gfm_server, &fd, &type,
+	    &inum, &gen, &real_url, cip);
+	if (e == GFARM_ERR_NO_ERROR) {
 		if (type != GFS_DT_REG) {
 			e = type == GFS_DT_DIR ? GFARM_ERR_IS_A_DIRECTORY :
 			    type == GFS_DT_LNK ? GFARM_ERR_IS_A_SYMBOLIC_LINK :
@@ -529,8 +657,9 @@ gfs_pio_open(const char *url, int flags, GFS_File *gfp)
 	GFARM_TIMEVAL_FIX_INITIALIZE_WARNING(t1);
 	gfs_profile(gfarm_gettimerval(&t1));
 
-	if ((e = gfm_open_fd(url, flags, &gfm_server, &fd, &type,
-	    &real_url, &ino, &gen, cip)) == GFARM_ERR_NO_ERROR) {
+	e = open_common(0, url, flags, 0, &gfm_server, &fd, &type,
+	    &ino, &gen, &real_url, cip);
+	if (e == GFARM_ERR_NO_ERROR) {
 		if (type != GFS_DT_REG) {
 			e = type == GFS_DT_DIR ? GFARM_ERR_IS_A_DIRECTORY :
 			    type == GFS_DT_LNK ? GFARM_ERR_IS_A_SYMBOLIC_LINK :
@@ -2295,61 +2424,73 @@ gfs_pio_file_list_foreach(struct gfs_file_list *gfl,
 	    GFS_FILE_LIST_MUTEX);
 }
 
+struct gfs_profile_list profile_items[] = {
+	{ "create_time", "gfs_pio_create time  : %g sec", "%g", 'd',
+	  offsetof(struct gfarm_gfs_pio_static, create_time) },
+	{ "create_count", "gfs_pio_create count : %llu", "%llu", 'l',
+	  offsetof(struct gfarm_gfs_pio_static, create_count) },
+	{ "open_time", "gfs_pio_open time   : %g sec", "%g", 'd',
+	  offsetof(struct gfarm_gfs_pio_static, open_time) },
+	{ "open_count", "gfs_pio_open count  : %llu", "%llu", 'l',
+	  offsetof(struct gfarm_gfs_pio_static, open_count) },
+	{ "close_time", "gfs_pio_close time  : %g sec", "%g", 'd',
+	  offsetof(struct gfarm_gfs_pio_static, close_time) },
+	{ "close_count", "gfs_pio_close count : %llu", "%llu", 'l',
+	  offsetof(struct gfarm_gfs_pio_static, close_count) },
+	{ "seek_time", "gfs_pio_seek time   : %g sec", "%g", 'd',
+	  offsetof(struct gfarm_gfs_pio_static, seek_time) },
+	{ "seek_count", "gfs_pio_seek count  : %llu", "%llu", 'l',
+	  offsetof(struct gfarm_gfs_pio_static, seek_count) },
+	{ "truncate_time", "gfs_pio_truncate time  : %g sec", "%g", 'd',
+	  offsetof(struct gfarm_gfs_pio_static, truncate_time) },
+	{ "truncate_count", "gfs_pio_truncate count : %llu", "%llu", 'l',
+	  offsetof(struct gfarm_gfs_pio_static, truncate_count) },
+	{ "read_time", "gfs_pio_read time   : %g sec", "%g", 'd',
+	  offsetof(struct gfarm_gfs_pio_static, read_time) },
+	{ "read_size", "gfs_pio_read size   : %llu", "%llu", 'l',
+	  offsetof(struct gfarm_gfs_pio_static, read_size) },
+	{ "read_count", "gfs_pio_read count  : %llu", "%llu", 'l',
+	  offsetof(struct gfarm_gfs_pio_static, read_count) },
+	{ "write_time", "gfs_pio_write time  : %g sec", "%g", 'd',
+	  offsetof(struct gfarm_gfs_pio_static, write_time) },
+	{ "write_size", "gfs_pio_write size  : %llu", "%llu", 'l',
+	  offsetof(struct gfarm_gfs_pio_static, write_size) },
+	{ "write_count", "gfs_pio_write count : %llu", "%llu", 'l',
+	  offsetof(struct gfarm_gfs_pio_static, write_count) },
+	{ "sync_time", "gfs_pio_sync time   : %g sec", "%g", 'd',
+	  offsetof(struct gfarm_gfs_pio_static, sync_time) },
+	{ "sync_count", "gfs_pio_sync count  : %llu", "%llu", 'l',
+	  offsetof(struct gfarm_gfs_pio_static, sync_count) },
+	{ "datasync_time", "gfs_pio_datasync time  : %g sec", "%g", 'd',
+	  offsetof(struct gfarm_gfs_pio_static, datasync_time) },
+	{ "datasync_count", "gfs_pio_datasync count : %llu", "%llu", 'l',
+	  offsetof(struct gfarm_gfs_pio_static, datasync_count) },
+	{ "getline_time", "gfs_pio_getline time  : %g sec (this calls getc)",
+	  "%g", 'd', offsetof(struct gfarm_gfs_pio_static, getline_time) },
+	{ "getline_count", "gfs_pio_getline count : %llu (this calls getc)",
+	  "%llu", 'l', offsetof(struct gfarm_gfs_pio_static, getline_count) },
+	{ "getc_time", "gfs_pio_getc time  : %g sec", "%g", 'd',
+	  offsetof(struct gfarm_gfs_pio_static, getc_time) },
+	{ "getc_count", "gfs_pio_getc count : %llu", "%llu", 'l',
+	  offsetof(struct gfarm_gfs_pio_static, getc_count) },
+	{ "putc_time", "gfs_pio_putc time  : %g sec", "%g", 'd',
+	  offsetof(struct gfarm_gfs_pio_static, putc_time) },
+	{ "putc_count",  "gfs_pio_putc count : %llu", "%llu", 'l',
+	  offsetof(struct gfarm_gfs_pio_static, putc_count) },
+};
+
 void
 gfs_pio_display_timers(void)
 {
-	gflog_info(GFARM_MSG_1000095,
-	    "gfs_pio_create time  : %g sec", staticp->create_time);
-	gflog_info(GFARM_MSG_1003807,
-	    "gfs_pio_create count : %llu", staticp->create_count);
-	gflog_info(GFARM_MSG_1000096,
-	    "gfs_pio_open time   : %g sec", staticp->open_time);
-	gflog_info(GFARM_MSG_1003808,
-	    "gfs_pio_open count  : %llu", staticp->open_count);
-	gflog_info(GFARM_MSG_1000097,
-	    "gfs_pio_close time  : %g sec", staticp->close_time);
-	gflog_info(GFARM_MSG_1003809,
-	    "gfs_pio_close count : %llu", staticp->close_count);
-	gflog_info(GFARM_MSG_1000098,
-	    "gfs_pio_seek time   : %g sec", staticp->seek_time);
-	gflog_info(GFARM_MSG_1003810,
-	    "gfs_pio_seek count  : %llu", staticp->seek_count);
-	gflog_info(GFARM_MSG_1000099,
-	    "gfs_pio_truncate time  : %g sec", staticp->truncate_time);
-	gflog_info(GFARM_MSG_1003811,
-	    "gfs_pio_truncate count : %llu", staticp->truncate_count);
-	gflog_info(GFARM_MSG_1000100,
-	    "gfs_pio_read time   : %g sec", staticp->read_time);
-	gflog_info(GFARM_MSG_1003812,
-	    "gfs_pio_read size   : %llu", staticp->read_size);
-	gflog_info(GFARM_MSG_1003813,
-	    "gfs_pio_read count  : %llu", staticp->read_count);
-	gflog_info(GFARM_MSG_1000101,
-	    "gfs_pio_write time  : %g sec", staticp->write_time);
-	gflog_info(GFARM_MSG_1003814,
-	    "gfs_pio_write size  : %llu", staticp->write_size);
-	gflog_info(GFARM_MSG_1003815,
-	    "gfs_pio_write count : %llu", staticp->write_count);
-	gflog_info(GFARM_MSG_1000102,
-	    "gfs_pio_sync time   : %g sec", staticp->sync_time);
-	gflog_info(GFARM_MSG_1003816,
-	    "gfs_pio_sync count  : %llu", staticp->sync_count);
-	gflog_info(GFARM_MSG_1003817,
-	    "gfs_pio_datasync time  : %g sec", staticp->datasync_time);
-	gflog_info(GFARM_MSG_1003818,
-	    "gfs_pio_datasync count : %llu", staticp->datasync_count);
-	gflog_info(GFARM_MSG_1000103,
-	    "gfs_pio_getline time  : %g sec (this calls getc)",
-			staticp->getline_time);
-	gflog_info(GFARM_MSG_1003819,
-	    "gfs_pio_getline count : %llu (this calls getc)",
-			staticp->getline_count);
-	gflog_info(GFARM_MSG_1000104,
-	    "gfs_pio_getc time  : %g sec", staticp->getc_time);
-	gflog_info(GFARM_MSG_1003820,
-	    "gfs_pio_getc count : %llu", staticp->getc_count);
-	gflog_info(GFARM_MSG_1000105,
-	    "gfs_pio_putc time  : %g sec", staticp->putc_time);
-	gflog_info(GFARM_MSG_1003821,
-	    "gfs_pio_putc count : %llu", staticp->putc_count);
+	int n = GFARM_ARRAY_LENGTH(profile_items);
+
+	gfs_profile_display_timers(n, profile_items, staticp);
+}
+
+gfarm_error_t
+gfs_pio_profile_value(const char *name, char *v, size_t *sp)
+{
+	int n = GFARM_ARRAY_LENGTH(profile_items);
+
+	return (gfs_profile_value(name, n, profile_items, staticp, v, sp));
 }
