@@ -1,10 +1,11 @@
 /*
  * pio operations for file fragments or programs
  *
- * $Id: gfs_pio_section.c 9937 2015-11-10 17:00:48Z n-soda $
+ * $Id: gfs_pio_section.c 11012 2019-02-08 15:18:57Z tatebe $
  */
 
 #include <assert.h>
+#include <stddef.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
@@ -126,7 +127,7 @@ gfs_pio_view_section_pwrite(GFS_File gf,
 				gf->mode &= ~GFS_FILE_MODE_DIGEST_CALC;
 			} else {
 				EVP_DigestUpdate(
-				    &gf->md_ctx, buffer, *lengthp);
+				    gf->md_ctx, buffer, *lengthp);
 				gf->md_offset += *lengthp;
 			}
 		}
@@ -158,7 +159,7 @@ gfs_pio_view_section_write(GFS_File gf,
 				gf->mode &= ~GFS_FILE_MODE_DIGEST_CALC;
 			} else {
 				EVP_DigestUpdate(
-				    &gf->md_ctx, buffer, *lengthp);
+				    gf->md_ctx, buffer, *lengthp);
 				gf->md_offset += *lengthp;
 			}
 		}
@@ -197,7 +198,7 @@ gfs_pio_view_section_pread(GFS_File gf,
 			 * AVAIL flags for consistency with the gfsd
 			 * implementation.
 			 */
-			EVP_DigestUpdate(&gf->md_ctx, buffer, *lengthp);
+			EVP_DigestUpdate(gf->md_ctx, buffer, *lengthp);
 			gf->md_offset += *lengthp;
 			if (gf->md_offset == gf->md.filesize && (gf->mode &
 			    GFS_FILE_MODE_MODIFIED) == 0)
@@ -223,7 +224,7 @@ gfs_pio_view_section_recvfile(GFS_File gf, gfarm_off_t r_off,
 		if (gf->md_offset != r_off) {
 			gf->mode &= ~GFS_FILE_MODE_DIGEST_CALC;
 		} else {
-			md_ctx = &gf->md_ctx;
+			md_ctx = gf->md_ctx;
 		}
 	}
 	e = (*vc->ops->storage_recvfile)(gf, r_off, w_fd, w_off, len,
@@ -267,7 +268,7 @@ gfs_pio_view_section_sendfile(GFS_File gf, gfarm_off_t w_off,
 		if (gf->md_offset != w_off) {
 			gf->mode &= ~GFS_FILE_MODE_DIGEST_CALC;
 		} else {
-			md_ctx = &gf->md_ctx;
+			md_ctx = gf->md_ctx;
 		}
 	}
 	e = (*vc->ops->storage_sendfile)(gf, w_off, r_fd, r_off, len,
@@ -279,7 +280,7 @@ gfs_pio_view_section_sendfile(GFS_File gf, gfarm_off_t w_off,
 		if (gf->md.filesize < w_off + sent)
 			gf->md.filesize = w_off + sent;
 	}
-	if (md_ctx != NULL) 
+	if (md_ctx != NULL)
 		gf->md_offset += sent;
 	if (sentp != NULL)
 		*sentp = sent;
@@ -302,8 +303,8 @@ gfs_pio_view_section_ftruncate(GFS_File gf, gfarm_off_t length)
 				if ((gf->mode & GFS_FILE_MODE_DIGEST_FINISH)
 				    == 0) {
 					/* to avoid memory leak */
-					gfarm_msgdigest_final(
-					    md_value, &gf->md_ctx);
+					gfarm_msgdigest_free(
+					    gf->md_ctx, md_value);
 					gf->mode |=
 					    GFS_FILE_MODE_DIGEST_FINISH;
 				}
@@ -862,7 +863,14 @@ gfs_pio_internal_set_view_section(GFS_File gf, char *host)
 		goto finish;
 #else /* not yet in gfarm v2  */
 		gf->mode |= GFS_FILE_MODE_CALC_DIGEST;
-		EVP_DigestInit(&vc->md_ctx, GFS_DEFAULT_DIGEST_MODE);
+		vc->md_ctx = gfarm_msgdigest_alloc(GFS_DEFAULT_DIGEST_MODE);
+		if (vc->md_ctx == NULL) {
+			e = GFARM_ERR_NO_MEMORY;
+			gflog_debug(GFARM_MSG_1004519,
+			    "msgdigest allocation failed: %s",
+			    gfarm_error_string(e));
+			goto finish;
+		}
 
 		if (gf->open_flags & GFARM_FILE_APPEND) {
 			e = gfs_pio_seek(gf, 0, SEEK_END, NULL);
@@ -1062,7 +1070,11 @@ gfs_pio_set_view_section(GFS_File gf, const char *section,
 	gf->io_offset = gf->offset = 0;
 
 	gf->mode |= GFS_FILE_MODE_CALC_DIGEST;
-	EVP_DigestInit(&vc->md_ctx, GFS_DEFAULT_DIGEST_MODE);
+	vc->md_ctx = gfarm_msgdigest_alloc(GFS_DEFAULT_DIGEST_MODE);
+	if (vc->md_ctx == NULL) {
+		e = GFARM_ERR_NO_MEMORY;
+		goto finish;
+	}
 
 	if (!is_local_host && gfarm_is_active_file_system_node &&
 	    (gf->mode & GFS_FILE_MODE_WRITE) == 0 &&
@@ -1147,7 +1159,7 @@ storage_close:
 		(void)(*vc->ops->storage_close)(gf);
 free_digest:
 	if (e != NULL)
-		EVP_DigestFinal(&vc->md_ctx, md_value, &md_len);
+		md_len = gfarm_msgdigest_free(vc->md_ctx, md_value);
 free_host:
 	if (e != GFARM_ERR_NO_ERROR)
 		free(vc->canonical_hostname);
@@ -1264,13 +1276,29 @@ gfarm_redirect_file(int fd, char *file, GFS_File *gfp)
 
 #endif /* not yet in gfarm v2 */
 
+struct gfs_profile_list section_profile_items[] = {
+	{ "set_view_section", "gfs_pio_set_view_section  : %g sec", "%g", 'd',
+	  offsetof(struct gfarm_gfs_pio_section_static, set_view_section_time)
+	},
+	{ "open_local_count", "gfs_pio_open_local_count  : %lld", "%llu", 'l',
+	  offsetof(struct gfarm_gfs_pio_section_static, open_local_count) },
+	{ "open_remote_count", "gfs_pio_open_remote_count : %lld", "%llu", 'l',
+	  offsetof(struct gfarm_gfs_pio_section_static, open_remote_count) },
+};
+
 void
 gfs_pio_section_display_timers(void)
 {
-	gflog_info(GFARM_MSG_1000113, "gfs_pio_set_view_section  : %f sec",
-		staticp->set_view_section_time);
-	gflog_info(GFARM_MSG_1003712, "gfs_pio_open_local_count  : %lld",
-		(unsigned long long)staticp->open_local_count);
-	gflog_info(GFARM_MSG_1003713, "gfs_pio_open_remote_count : %lld",
-		(unsigned long long)staticp->open_remote_count);
+	int n = GFARM_ARRAY_LENGTH(section_profile_items);
+
+	gfs_profile_display_timers(n, section_profile_items, staticp);
+}
+
+gfarm_error_t
+gfs_pio_section_profile_value(const char *name, char *value, size_t *sizep)
+{
+	int n = GFARM_ARRAY_LENGTH(section_profile_items);
+
+	return (gfs_profile_value(name, n, section_profile_items,
+		    staticp, value, sizep));
 }
