@@ -1,5 +1,5 @@
 /*
- * $Id: liberror.c 10524 2017-06-16 01:37:17Z n-soda $
+ * $Id: liberror.c 11265 2019-08-21 10:38:09Z tatebe $
  */
 
 #include <pthread.h>
@@ -192,6 +192,7 @@ static const char *errmsg_string[GFARM_ERRMSG_END - GFARM_ERRMSG_BEGIN] = {
 	"local group name redefined",
 	"global group name redefined",
 	"missing second field (local user)",
+	"db_access_type is none but backend database is specified",
 	"inconsistent configuration, LDAP is specified as metadata backend before",
 	"inconsistent configuration, PostgreSQL is specified as metadata backend before",
 	"inconsistent configuration, localfs is specified as metadata backend before",
@@ -790,7 +791,10 @@ gfarm_error_to_errno(gfarm_error_t error)
 
 	pthread_once(&gfarm_error_to_errno_initialized,
 	    gfarm_error_to_errno_initialize);
-	if (error < GFARM_ERR_NUMBER)
-		return (gfarm_error_to_errno_map[error]);
-	return (EINVAL);
+	if (error < 0 || error >= GFARM_ERR_NUMBER) {
+		gflog_warning(GFARM_MSG_1005090,
+		    "gfarm_error_to_errno: invalid error: %d", error);
+		return (EINVAL);
+	}
+	return (gfarm_error_to_errno_map[error]);
 }
