@@ -1,10 +1,7 @@
 /*
- * $Id: repattr.h 7590 2013-01-10 05:59:59Z takuya-i $
+ * $Id: repattr.h 10955 2019-01-15 12:48:37Z n-soda $
  */
 
-#define GFARM_REPATTR_TYPE	"replicainfo"
-#define GFARM_REPATTR_NAME	"gfarm." GFARM_REPATTR_TYPE
-
 struct gfarm_repattr;
 typedef struct gfarm_repattr *gfarm_repattr_t;
 
