@@ -1,5 +1,5 @@
 /*
- * $Id: timer.c 10137 2016-04-23 03:10:39Z tatebe $
+ * $Id: timer.c 10659 2017-11-08 00:58:00Z n-soda $
  *
  * TIMER routine
  *
@@ -19,6 +19,7 @@
 
 #if 0
 
+#include <pthread.h>
 #include <unistd.h>
 
 double gfarm_timerval_calibration;
@@ -32,8 +33,8 @@ gfarm_get_cycles(void)
 	return (rv);
 }
 
-void
-gfarm_timerval_calibrate(void)
+static void
+gfarm_timerval_calibrate_once(void)
 {
 	gfarm_timerval_t t1, t2;
 	struct timeval s1, s2;
@@ -54,6 +55,14 @@ gfarm_timerval_calibrate(void)
 		(t2 - t1);
 }
 
+void
+gfarm_timerval_calibrate(void)
+{
+	static pthread_once_t calibrated = PTHREAD_ONCE_INIT;
+
+	pthread_once(&calibrated, gfarm_timerval_calibrate_once);
+}
+
 #else /* gettimeofday */
 
 void
