@@ -7,11 +7,11 @@ GFNCOPY_TIMEOUT=60
 
 #####################################################################
 
-dir=/tmp/autoreplica-test.$$
+dir=$gftmp/autoreplica-test.$$
 file=${dir}/file.$$
 gfs_pio_test=`dirname $testbin`/gfs_pio_test/gfs_pio_test
-tmpfile=/tmp/.autoreplica-test.junk.$$
-hostgroupfile=/tmp/.hostgroup.$$
+tmpfile=$localtop/.autoreplica-test.junk.$$
+hostgroupfile=$localtop/.hostgroup.$$
 
 mod="over written"
 
@@ -44,7 +44,7 @@ cleanup() {
     gfncopy -r ${file} > /dev/null 2>&1
     gfrm -f ${file} > /dev/null 2>&1
     gfncopy -r ${dir} > /dev/null 2>&1
-    gfrmdir ${dir} > /dev/null 2>&1
+    gfrm -r ${gftmp} > /dev/null 2>&1
     for __i in ${hosts}; do
 	gfhostgroup -r ${__i} > /dev/null 2>&1
     done
@@ -70,7 +70,7 @@ onexit() {
 }
 
 setup() {
-    gfmkdir ${dir}
+    gfmkdir -p ${dir}
     ret=$?
     if [ ${ret} -ne 0 ]; then
 	return ${ret}
