@@ -3022,7 +3022,7 @@ t_apply_xattr_add_cached(void)
 {
 	struct inode *i;
 	struct db_xattr_arg m;
-	const char *attrname = "gfarm.ncopy";
+	const char *attrname = GFARM_EA_NCOPY;
 	const char *value = "2";
 	void *value1;
 	size_t value_len;
@@ -3076,7 +3076,7 @@ t_apply_xattr_modify_cached(void)
 {
 	struct inode *i;
 	struct db_xattr_arg m;
-	const char *attrname = "gfarm.ncopy";
+	const char *attrname = GFARM_EA_NCOPY;
 	const char *value = "3";
 	void *value1;
 	size_t value_len;
@@ -3126,7 +3126,7 @@ t_apply_xattr_remove_cached(void)
 {
 	struct inode *i;
 	struct db_xattr_arg m;
-	const char *attrname = "gfarm.ncopy";
+	const char *attrname = GFARM_EA_NCOPY;
 	void *value;
 	size_t value_len;
 
@@ -3152,7 +3152,7 @@ t_apply_xattr_removeall(void)
 	struct inode *i;
 	struct db_xattr_arg m;
 	const char *attrname1 = "attrname";
-	const char *attrname2 = "gfarm.ncopy";
+	const char *attrname2 = GFARM_EA_NCOPY;
 	const char *value1 = "value";
 	const char *value2 = "2";
 	void *value;
@@ -3421,10 +3421,12 @@ t_apply_mdhost_add(void)
 	struct gfarm_metadb_server ms;
 	struct mdhost *h;
 
+	memset(&ms, 0, sizeof(ms));
 	ms.name = assert_strdup(name);
 	ms.clustername = assert_strdup("clustername");
 	ms.port = 1234;
 	ms.flags = 2;
+	gfarm_metadb_server_set_is_memory_owned_by_fs(&ms, 0);
 
 	TEST_ASSERT_B("mdhost_lookup",
 	    (h = mdhost_lookup(name)) == NULL);
@@ -3459,6 +3461,7 @@ t_apply_mdhost_modify(void)
 	m.ms.port = 5678;
 	m.ms.flags = 5;
 	m.modflags = 0; /* modflags is not used yet */
+	gfarm_metadb_server_set_is_memory_owned_by_fs(&m.ms, 0);
 
 	TEST_ASSERT_NOERR("mdhost_modify",
 	    db_journal_apply_ops.mdhost_modify(0, &m));
@@ -3609,13 +3612,20 @@ t_apply(void)
 int
 main(int argc, char **argv)
 {
+	gfarm_error_t e;
 	int c, op = 0;
 
 	/* XXX: settings in gfmd.conf doesn't work in this case */
 	char *config  = getenv("GFARM_CONFIG_FILE");
 
 	debug_mode = 1;
-	gfarm_server_initialize(config, &argc, &argv);
+	e = gfarm_server_initialize(config, &argc, &argv);
+	if (e != GFARM_ERR_NO_ERROR) {
+		fprintf(stderr, "%s: gfarm_server_initialize: %s\n",
+		    argv[0], gfarm_error_string(e));
+		fprintf(stderr, "%s: aborting\n", argv[0]);
+		exit(EXIT_FAILURE);
+	}
 	gflog_set_priority_level(LOG_DEBUG);
 	gflog_set_message_verbose(99);
 
