@@ -15,7 +15,7 @@
 /*
  * Metadata access switch for internal implementation
  *
- * $Id: db_ops.h 9743 2015-06-21 23:10:13Z n-soda $
+ * $Id: db_ops.h 10309 2016-11-30 23:32:30Z n-soda $
  */
 
 struct gfarm_internal_host_info;
@@ -34,11 +34,13 @@ struct db_fsngroup_modify_arg {
 
 struct db_user_modify_arg {
 	struct gfarm_user_info ui;
+		/* should be first member, for db_user_dup() */
 	int modflags;
 };
 
 struct db_group_modify_arg {
 	struct gfarm_group_info gi;
+		/* should be first member, for db_group_dup() */
 	int modflags;
 	int add_count; char **add_users;
 	int del_count; char **del_users;
@@ -129,13 +131,25 @@ struct db_quota_remove_arg {
 	char *name;
 };
 
+struct gfarm_quota_info;
+
+struct db_quota_dirset_arg {
+	struct gfarm_dirset_info dirset;
+		/* should be first member, for db_dirset_dup()*/
+	struct quota_metadata q;
+};
+
+struct db_inode_dirset_arg {
+	struct gfarm_dirset_info dirset;
+		/* should be first member, for db_dirset_dup()*/
+	gfarm_ino_t inum;
+};
+
 struct db_seqnum_arg {
 	char *name;
 	gfarm_uint64_t value;
 };
 
-struct gfarm_quota_info;
-
 struct db_mdhost_modify_arg {
 	struct gfarm_metadb_server ms;
 	int modflags;
@@ -252,6 +266,23 @@ struct db_ops {
 	gfarm_error_t (*quota_load)(void *, int,
 		void (*)(void *, struct gfarm_quota_info *));
 
+	gfarm_error_t (*quota_dirset_add)(gfarm_uint64_t,
+		struct db_quota_dirset_arg *);
+	gfarm_error_t (*quota_dirset_modify)(gfarm_uint64_t,
+		struct db_quota_dirset_arg *);
+	gfarm_error_t (*quota_dirset_remove)(gfarm_uint64_t,
+		struct gfarm_dirset_info *);
+	gfarm_error_t (*quota_dirset_load)(void *,
+		void (*)(void *,
+		    struct gfarm_dirset_info *, struct quota_metadata *));
+
+	gfarm_error_t (*quota_dir_add)(gfarm_uint64_t,
+		struct db_inode_dirset_arg *);
+	gfarm_error_t (*quota_dir_remove)(gfarm_uint64_t,
+		struct db_inode_inum_arg *);
+	gfarm_error_t (*quota_dir_load)(void *,
+	    void (*)(void *, gfarm_ino_t, struct gfarm_dirset_info *));
+
 	gfarm_error_t (*seqnum_get)(const char *, gfarm_uint64_t *);
 	gfarm_error_t (*seqnum_add)(struct db_seqnum_arg *);
 	gfarm_error_t (*seqnum_modify)(struct db_seqnum_arg *);
