@@ -1,5 +1,5 @@
 /*
- * $Id: fsngroup.h 10665 2017-11-13 10:04:19Z takuya-i $
+ * $Id: fsngroup.h 10951 2019-01-15 12:43:42Z n-soda $
  */
 
 struct host;
@@ -11,11 +11,14 @@ gfarm_error_t fsngroup_get_hosts(const char *, int *, struct host ***);
 struct inode;
 struct dirset;
 struct file_copy;
+struct hostset;
 gfarm_error_t fsngroup_schedule_replication(
 	struct inode *, struct dirset *tdirset,
 	const char *, int, struct host **,
-	int *, struct host **, gfarm_time_t, int *, struct host **,
+	int *, struct hostset *, gfarm_time_t, int *, struct hostset *,
 	const char *, int *, int *);
+gfarm_error_t fsngroup_has_spare_for_repattr(struct inode *, int,
+	const char *, const char *, int);
 
 /*
  * Server side RPC stubs:
