@@ -1,5 +1,5 @@
 /*
- * $Id: gfmd.c 10766 2018-02-06 06:04:05Z n-soda $
+ * $Id: gfmd.c 11265 2019-08-21 10:38:09Z tatebe $
  */
 
 #include <gfarm/gfarm_config.h>
@@ -42,7 +42,6 @@
 #include "auth.h"
 #include "config.h"
 #include "gfm_proto.h"
-#include "repattr.h"
 
 #include "subr.h"
 #include "thrpool.h"
@@ -447,6 +446,10 @@ protocol_switch(struct peer *peer, int from_client, int skip, int level,
 	case GFM_PROTO_CONFIG_SET:
 		e = gfm_server_config_set(peer, from_client, skip);
 		break;
+	case GFM_PROTO_SCHEDULE_HOST_DOMAIN_USE_REAL_DISK_SPACE:
+		e = gfm_server_schedule_host_domain_use_real_disk_space(peer,
+		    from_client, skip);
+		break;
 	case GFM_PROTO_REPLICA_LIST_BY_NAME:
 		e = gfm_server_replica_list_by_name(peer, from_client, skip);
 		break;
@@ -995,8 +998,12 @@ peer_authorize(struct peer *peer)
 	rv = getpeername(gfp_xdr_fd(peer_get_conn(peer)), &addr, &addrlen);
 	if (rv == -1) {
 		saved_errno = errno;
-		gflog_error(GFARM_MSG_1000184,
-		    "authorize: getpeername: %s", strerror(errno));
+		if (saved_errno == ENOTCONN) /* known race condition */
+			gflog_notice(GFARM_MSG_1005070,
+			    "authorize: getpeername: %s", strerror(errno));
+		else
+			gflog_error(GFARM_MSG_1000184,
+			    "authorize: getpeername: %s", strerror(errno));
 		return (gfarm_errno_to_error(saved_errno));
 	}
 	e = gfarm_sockaddr_to_name(&addr, &hostname);
@@ -1005,7 +1012,7 @@ peer_authorize(struct peer *peer)
 		    addr_string, GFARM_SOCKADDR_STRLEN);
 		gflog_info(GFARM_MSG_1000185,
 		    "gfarm_sockaddr_to_name(%s): %s",
-		    gfarm_error_string(e), addr_string);
+		    addr_string, gfarm_error_string(e));
 		hostname = strdup_log(addr_string, diag);
 		if (hostname == NULL)
 			return (GFARM_ERR_NO_MEMORY);
@@ -1538,6 +1545,7 @@ usage(void)
 {
 	fprintf(stderr, "Usage: %s [option]\n", program_name);
 	fprintf(stderr, "option:\n");
+	fprintf(stderr, "\t-F <log-file>\n");
 	fprintf(stderr, "\t-L <syslog-priority-level>\n");
 	fprintf(stderr, "\t-P <pid-file>\n");
 	fprintf(stderr, "\t-d\t\t\t\t... debug mode\n");
@@ -1689,6 +1697,7 @@ main(int argc, char **argv)
 	char *config_file = NULL, *port_number = NULL;
 	int syslog_level = -1;
 	int syslog_facility = GFARM_DEFAULT_FACILITY;
+	char *syslog_file = NULL;
 	int ch, sock, table_size;
 	sigset_t sigs;
 	int is_master, replication_enabled;
@@ -1698,8 +1707,11 @@ main(int argc, char **argv)
 		program_name = basename(argv[0]);
 	gflog_set_identifier(program_name);
 
-	while ((ch = getopt(argc, argv, "L:P:df:p:Ss:tv")) != -1) {
+	while ((ch = getopt(argc, argv, "F:L:P:df:p:Ss:tv")) != -1) {
 		switch (ch) {
+		case 'F':
+			syslog_file = optarg;
+			break;
 		case 'L':
 			syslog_level = gflog_syslog_name_to_priority(optarg);
 			if (syslog_level == -1)
@@ -1799,7 +1811,12 @@ main(int argc, char **argv)
 	}
 
 	if (!debug_mode) {
-		gflog_syslog_open(LOG_PID, syslog_facility);
+		if (syslog_file != NULL) {
+			if (gflog_file_open(syslog_file) == NULL)
+				gflog_fatal_errno(GFARM_MSG_1005100,
+				    "%s", syslog_file);
+		} else
+			gflog_syslog_open(LOG_PID, syslog_facility);
 		if (gfarm_daemon(0, 0) == -1)
 			gflog_warning_errno(GFARM_MSG_1001487, "daemon");
 	}
@@ -1929,8 +1946,10 @@ main(int argc, char **argv)
 			sock = open_accepting_socket(gfmd_port);
 		else
 			sock = wait_transform_to_master(gfmd_port);
-	} else
+	} else {
+		gflog_info(GFARM_MSG_1005101, "metadata replication disabled");
 		sock = open_accepting_socket(gfmd_port);
+	}
 
 	/* master */
 
