@@ -49,8 +49,9 @@ void inode_set_group_by_name_in_cache(struct inode *, const char *);
 int inode_has_no_replica(struct inode *);
 gfarm_int64_t inode_get_ncopy(struct inode *);
 gfarm_int64_t inode_get_ncopy_with_dead_host(struct inode *);
-gfarm_error_t inode_count_replicas(
-	struct inode *, int, int, gfarm_time_t, int, struct host **, int *);
+struct hostset;
+int inode_count_replicas_within_scope(
+	struct inode *, int, int, gfarm_time_t, struct hostset *);
 
 gfarm_mode_t inode_get_mode(struct inode *);
 gfarm_error_t inode_set_mode(struct inode *, gfarm_mode_t);
@@ -141,22 +142,18 @@ void inode_remove_replica_in_cache_for_invalid_host(gfarm_ino_t);
 int inode_is_updated(struct inode *, struct gfarm_timespec *);
 gfarm_error_t dir_entry_add(gfarm_ino_t, char *, int, gfarm_ino_t);
 
+struct hostset;
 gfarm_error_t inode_schedule_replication_within_scope(
 	struct inode *, struct dirset *, int,
 	int, struct host **, int *,
-	int *, struct host **,
-	int *, struct host **, gfarm_time_t,
-	int *, struct host **, const char *, int *);
-gfarm_error_t inode_schedule_replication_from_all(
-	struct inode *, struct dirset *, int,
-	int, struct host **,
-	int *, struct host **, gfarm_time_t,
-	int *, struct host **, const char *, int *);
+	int *, struct hostset *,
+	int *, struct hostset *, gfarm_time_t,
+	int *, struct hostset *, const char *, int *);
 gfarm_error_t inode_schedule_replication(
 	struct inode *, struct dirset *, int, int, const char *,
 	int, struct host **,
-	int *, struct host **, gfarm_time_t,
-	int *, struct host **, const char *, int *);
+	int *, struct hostset *, gfarm_time_t,
+	int *, struct hostset *, const char *, int *);
 
 gfarm_error_t inode_open(struct file_opening *, struct dirset *);
 struct dirset *inode_get_tdirset(struct inode *);
@@ -197,6 +194,7 @@ gfarm_uint64_t inode_get_open_status_by_host(struct inode *, struct host *);
 struct file_copy * inode_get_file_copy(struct inode *, struct host *);
 int inode_has_file_copy(struct inode *, struct host *);
 int inode_has_replica(struct inode *, struct host *);
+int inode_has_writable_replica(struct inode *);
 gfarm_error_t inode_getdirpath(struct inode *, struct process *, char **);
 struct host *inode_schedule_host_for_read(struct inode *, struct host *);
 struct host *inode_schedule_host_for_write(struct inode *, struct host *);
@@ -211,11 +209,6 @@ extern gfarm_error_t (*inode_schedule_file)(struct file_opening *,
 
 struct file_replicating;
 void replication_info(void);
-gfarm_error_t file_replicating_new(
-	struct inode *, struct host *, struct dead_file_copy *,
-	struct dirset *, struct file_replicating **);
-void file_replicating_free(struct file_replicating *);
-void file_replicating_free_by_error_before_request(struct file_replicating *);
 gfarm_int64_t file_replicating_get_gen(struct file_replicating *);
 gfarm_error_t inode_replicated(struct file_replicating *,
 	gfarm_int32_t, gfarm_int32_t, gfarm_off_t,
@@ -225,12 +218,12 @@ gfarm_error_t inode_prepare_to_replicate(struct inode *, struct user *,
 	struct file_replicating **);
 void inode_replication_get_cksum_mode(struct inode *, struct host *,
 	char **, size_t *, char **, gfarm_int32_t *);
-gfarm_error_t inode_replication_request(struct host *, struct host *,
+gfarm_error_t inode_replication_request(
 	struct inode *, struct file_replicating *, const char *);
 
-gfarm_error_t inode_replica_hosts(
-	struct inode *, gfarm_int32_t *, struct host ***,
-	gfarm_int32_t *, struct host ***);
+gfarm_error_t inode_replica_hostset(
+	struct inode *, gfarm_int32_t *, struct hostset **,
+	gfarm_int32_t *, struct hostset **);
 gfarm_error_t inode_replica_hosts_valid(struct inode *,
 	gfarm_int32_t *, struct host ***);
 
