@@ -689,13 +689,13 @@ mdhost_set_self_as_master(void)
 }
 
 gfarm_error_t
-mdhost_enter(struct gfarm_metadb_server *ms, struct mdhost **mpp)
+mdhost_enter_internal(struct gfarm_metadb_server *ms, struct mdhost **mpp)
 {
 	struct gfarm_hash_entry *entry;
 	int created;
 	struct mdhost *mh;
 	gfarm_error_t e;
-	static const char diag[] = "mdhost_enter";
+	static const char diag[] = "mdhost_enter_internal";
 
 	mh = mdhost_lookup_internal(ms->name);
 	if (mh) {
@@ -754,6 +754,20 @@ mdhost_enter(struct gfarm_metadb_server *ms, struct mdhost **mpp)
 	return (GFARM_ERR_NO_ERROR);
 }
 
+gfarm_error_t
+mdhost_enter(struct gfarm_metadb_server *ms, struct mdhost **mpp)
+{
+	gfarm_error_t e = mdhost_enter_internal(ms, mpp);
+
+	if (e != GFARM_ERR_NO_ERROR)
+		return (e);
+
+	if ((e = mdhost_updated()) != GFARM_ERR_NO_ERROR)
+		gflog_error(GFARM_MSG_1005079,
+		    "%s", gfarm_error_string(e));
+	return (e);
+}
+
 int
 mdhost_is_sync_replication(struct mdhost *mh)
 {
@@ -795,14 +809,16 @@ mdhost_has_async_replication_target(void)
 	return (ret);
 }
 
-void
+static void
 mdhost_add_one(void *closure, struct gfarm_metadb_server *ms)
 {
-	gfarm_error_t e = mdhost_enter(ms, NULL);
+	gfarm_error_t e = mdhost_enter_internal(ms, NULL);
 
 	if (e != GFARM_ERR_NO_ERROR)
 		gflog_warning(GFARM_MSG_1002932,
 		    "mdhost_add_one: %s", gfarm_error_string(e));
+
+	/* mdhost_updated() will be called by mdhost_init() */
 }
 
 static gfarm_error_t
@@ -916,14 +932,15 @@ mdhost_update_replication_type(struct mdhost *mh,
 	return (e);
 }
 
-gfarm_error_t
-mdhost_modify_in_cache(struct mdhost *mh, struct gfarm_metadb_server *ms)
+static gfarm_error_t
+mdhost_modify_in_cache_internal(
+	struct mdhost *mh, struct gfarm_metadb_server *ms)
 {
 	int cluster_changed;
 	gfarm_error_t e = GFARM_ERR_NO_ERROR;
 	char *old_clustername = NULL;
 	char *new_clustername;
-	static const char diag[] = "mdhost_modify_in_cache";
+	static const char diag[] = "mdhost_modify_in_cache_internal";
 
 	mdhost_mutex_lock(mh, diag);
 	cluster_changed = strcmp(mh->ms.clustername, ms->clustername) != 0;
@@ -957,6 +974,20 @@ end:
 	return (e);
 }
 
+gfarm_error_t
+mdhost_modify_in_cache(struct mdhost *mh, struct gfarm_metadb_server *ms)
+{
+	gfarm_error_t e = mdhost_modify_in_cache_internal(mh, ms);
+
+	if (e != GFARM_ERR_NO_ERROR)
+		return (e);
+
+	if ((e = mdhost_updated()) != GFARM_ERR_NO_ERROR)
+		gflog_error(GFARM_MSG_1005080,
+		    "%s", gfarm_error_string(e));
+	return (e);
+}
+
 /* PREREQUISITE: giant_lock */
 gfarm_error_t
 mdhost_remove_in_cache(const char *name)
@@ -1272,7 +1303,7 @@ mdhost_updated(void)
 	}
 	mdhost_table_rwlock_unlock(diag);
 	fs = gfarm_filesystem_get_default();
-	e = gfarm_filesystem_set_metadb_server_list(fs, mss, i);
+	e = gfarm_filesystem_replace_metadb_server_list(fs, mss, i);
 	if (e != GFARM_ERR_NO_ERROR)
 		gflog_error(GFARM_MSG_1004730,
 		    "%s: gfarm_filesystem_set_metadb_server_list: %s",
@@ -1374,7 +1405,8 @@ gfm_server_metadb_server_set(struct peer *peer, int from_client, int skip)
 	} else if ((e = metadb_server_verify(&ms, diag))
 	    != GFARM_ERR_NO_ERROR) {
 		/* nothing to do */
-	} else if ((e = mdhost_enter(&ms, &mh)) != GFARM_ERR_NO_ERROR) {
+	} else if ((e = mdhost_enter_internal(&ms, &mh))
+	    != GFARM_ERR_NO_ERROR) {
 		/* nothing to do */
 	} else if (gfarm_metadb_server_is_default_master(&ms)) {
 		if ((e = db_begin(diag)) != GFARM_ERR_NO_ERROR) {
@@ -1465,7 +1497,7 @@ gfm_server_metadb_server_modify(struct peer *peer, int from_client, int skip)
 		   != GFARM_ERR_NO_ERROR) {
 		/* nothing to do */
 	} else {
-		mdhost_modify_in_cache(mh, &ms);
+		mdhost_modify_in_cache_internal(mh, &ms);
 		if (isdm) {
 			e = mdhost_db_modify_default_master(mh, &ms, diag);
 			if (e != GFARM_ERR_NO_ERROR)
@@ -1582,7 +1614,8 @@ mdhost_initial_entry(void)
 		gfarm_metadb_server_set_is_master(&ms, 1);
 		gfarm_metadb_server_set_is_master_candidate(&ms, 1);
 		gfarm_metadb_server_set_is_default_master(&ms, 1);
-		if ((e = mdhost_enter(&ms, &self)) != GFARM_ERR_NO_ERROR)
+		if ((e = mdhost_enter_internal(&ms, &self))
+		    != GFARM_ERR_NO_ERROR)
 			gflog_fatal(GFARM_MSG_1002972,
 			    "Failed to add self mdhost");
 		else if (gfarm_get_metadb_replication_enabled()) {
