@@ -975,6 +975,14 @@ process_close_or_abort_file(struct process *process, struct peer *peer, int fd,
 			return (GFARM_ERR_NO_ERROR);
 		}
 	} else {
+		if ((accmode_to_op(fo->flag) & GFS_W_OK) != 0 && aborted) {
+			gflog_info(GFARM_MSG_1005068,
+			    "(%s@%s) aborted without closing a write-opened "
+			    "file %llu:%llu",
+			    peer_get_username(peer), peer_get_hostname(peer),
+			    (unsigned long long)inode_get_number(fo->inode),
+			    (unsigned long long)inode_get_gen(fo->inode));
+		}
 		if (GFARM_S_ISREG(mode) &&
 		    fo->u.f.spool_opener != NULL &&
 		    fo->u.f.spool_opener != peer) {
@@ -1291,7 +1299,7 @@ process_replication_request(struct process *process, struct peer *peer,
 
 	if ((e = process_prepare_to_replicate(process, peer, src, dst,
 	    fd, flags, &fr, &inode, diag)) == GFARM_ERR_NO_ERROR)
-		e = inode_replication_request(src, dst, inode, fr, diag);
+		e = inode_replication_request(inode, fr, diag);
 	return (e);
 }
 
