@@ -1,5 +1,5 @@
 /*
- * $Id: replica_check.c 10774 2018-02-13 13:07:50Z n-soda $
+ * $Id: replica_check.c 11125 2019-03-22 12:27:08Z n-soda $
  */
 
 #include <stdio.h>
@@ -27,7 +27,6 @@
 
 #include "config.h"
 #include "auth.h"
-#include "repattr.h"
 #include "gfm_proto.h"
 #include "timespec.h"
 
@@ -259,6 +258,10 @@ replica_check_remove_replicas(struct inode *inode,
 
 	/* prefer a host which has few disk_avail */
 	host_sort_to_remove_replicas(n_srcs, srcs);
+	/*
+	 * disk_avail of readonly-host is 0, but replicas on
+	 * readonly-host is not removed.
+	 */
 
 	/*
 	 * gfarm_replica_check_remove_grace_used_space_ratio and
@@ -349,7 +352,8 @@ replica_check_fix(struct replication_info *info)
 	gfarm_error_t e;
 	struct inode *inode = inode_lookup(info->inum);
 	int n_srcs, n_existing, n_being_removed, transaction = 0;
-	struct host **srcs, **existing, **being_removed;
+	struct host **srcs;
+	struct hostset *existing, *being_removed;
 	int req_ok_num = 0;
 	static const char diag[] = "replica_check_fix";
 
@@ -369,7 +373,7 @@ replica_check_fix(struct replication_info *info)
 		return (GFARM_ERR_NO_ERROR); /* ignore */
 	}
 
-	e = inode_replica_hosts(
+	e = inode_replica_hostset(
 	    inode, &n_existing, &existing, &n_being_removed, &being_removed);
 	if (e != GFARM_ERR_NO_ERROR) { /* no memory */
 		gflog_error(GFARM_MSG_1003692,
@@ -379,8 +383,8 @@ replica_check_fix(struct replication_info *info)
 		return (e); /* retry */
 	}
 	if (n_existing == 0) {
-		free(existing);
-		free(being_removed);
+		hostset_free(existing);
+		hostset_free(being_removed);
 		if (inode_get_size(inode) == 0)
 			return (GFARM_ERR_NO_ERROR); /* normally */
 		gflog_error(GFARM_MSG_1003624,
@@ -393,8 +397,8 @@ replica_check_fix(struct replication_info *info)
 	/* available replicas for source */
 	e = inode_replica_hosts_valid(inode, &n_srcs, &srcs);
 	if (e != GFARM_ERR_NO_ERROR) { /* no memory */
-		free(existing);
-		free(being_removed);
+		hostset_free(existing);
+		hostset_free(being_removed);
 		gflog_error(GFARM_MSG_1003628,
 		    "replica_check: %lld:%lld:%s: replica_list: %s",
 		    (long long)info->inum, (long long)info->gen,
@@ -404,8 +408,8 @@ replica_check_fix(struct replication_info *info)
 
 	/* n_srcs may be 0, because host_is_up() may change */
 	if (n_srcs <= 0) { /* hosts are down, no available replica */
-		free(existing);
-		free(being_removed);
+		hostset_free(existing);
+		hostset_free(being_removed);
 		free(srcs);
 		REDUCED_WARN(GFARM_MSG_1004274, &hosts_down_state,
 		    "replica_check: %lld:%lld:%s: no available replica",
@@ -416,8 +420,8 @@ replica_check_fix(struct replication_info *info)
 
 	if (info->replica_spec.repattr == NULL &&
 	    info->replica_spec.desired_number <= 0) { /* disabled */
-		free(existing);
-		free(being_removed);
+		hostset_free(existing);
+		hostset_free(being_removed);
 		free(srcs);
 		return (GFARM_ERR_NO_ERROR); /* skip */
 	}
@@ -446,8 +450,8 @@ replica_check_fix(struct replication_info *info)
 
 	if (transaction)
 		db_end(diag);
-	free(existing);
-	free(being_removed);
+	hostset_free(existing);
+	hostset_free(being_removed);
 	free(srcs);
 	return (e);
 }
@@ -721,6 +725,8 @@ replica_check_main(void)
 	    replica_check_remove_grace_used_space_ratio_locked(),
 	    replica_check_remove_grace_time_locked());
 
+	replication_info(); /* acquires giant_lock internally */
+
 	for (inum = root_inum;;) {
 		if (inum % REPLICA_CHECK_INTERRUPT_STEP == 0 &&
 		    !replica_check_ctrl_enabled()) { /* gfrepcheck stop */
@@ -765,6 +771,7 @@ replica_check_main(void)
 		    "replica_check: yield sleep=%g",
 		    total_yield_time);
 	}
+	replication_info(); /* acquires giant_lock internally */
 
 	replica_check_giant_lock();
 	info_time_start = 0;  /* stopped */
@@ -948,7 +955,7 @@ replica_check_cond_wait(void)
 {
 	static const char diag[] = "replica_check_cond_wait";
 	struct timeval next, now;
-	int minimum_interval = replica_check_minimum_interval_locked();
+	int minimum_interval = replica_check_minimum_interval_locked(), t;
 
 	gfarm_mutex_lock(&replica_check_mutex, diag, REPLICA_CHECK_DIAG);
 	/* do not call giant_lock() in replica_check_mutex */
@@ -989,8 +996,18 @@ replica_check_cond_wait(void)
 		 */
 		gfarm_mutex_unlock(
 		    &replica_check_mutex, diag, REPLICA_CHECK_DIAG);
-		gfarm_nanosleep((unsigned long long)minimum_interval *
-		    GFARM_SECOND_BY_NANOSEC);
+
+		t = minimum_interval;
+		while (t > 0) {
+			gfarm_sleep(1);
+			t -= 1;
+			/* use latest minimum_interval */
+			minimum_interval =
+				replica_check_minimum_interval_locked();
+			if (t > minimum_interval)
+				t = minimum_interval;
+		}
+
 		gfarm_mutex_lock(
 		    &replica_check_mutex, diag, REPLICA_CHECK_DIAG);
 	}
@@ -1097,6 +1114,34 @@ static int replica_check_ctrl_waiting = 0;
 static int replica_check_ctrl = DISABLE;
 static int replica_check_remove = DISABLE;
 static int replica_check_reduced_log = DISABLE;
+static int is_running = 0;
+
+static int
+replica_check_is_running(void)
+{
+	static const char diag[] = "replica_check_is_running";
+	int is;
+
+	gfarm_mutex_lock(&replica_check_ctrl_mutex, diag,
+	    REPLICA_CHECK_CTRL_DIAG);
+	is = is_running;
+	gfarm_mutex_unlock(&replica_check_ctrl_mutex, diag,
+	    REPLICA_CHECK_CTRL_DIAG);
+
+	return (is);
+}
+
+static void
+replica_check_is_running_update(int is)
+{
+	static const char diag[] = "replica_check_is_running_update";
+
+	gfarm_mutex_lock(&replica_check_ctrl_mutex, diag,
+	    REPLICA_CHECK_CTRL_DIAG);
+	is_running = is;
+	gfarm_mutex_unlock(&replica_check_ctrl_mutex, diag,
+	    REPLICA_CHECK_CTRL_DIAG);
+}
 
 /* MAINCTRL */
 static void
@@ -1110,6 +1155,8 @@ replica_check_ctrl_set(int ctrl)
 	/* escape from replica_check_cond_wait() */
 	if (replica_check_ctrl_waiting == 0)
 		replica_check_cond_signal(diag, 0);
+	if (ctrl == ENABLE)
+		is_running = 1;
 
 	replica_check_ctrl = ctrl;
 	gfarm_cond_signal(&replica_check_ctrl_cond, diag,
@@ -1300,13 +1347,13 @@ replica_check_status_mainctrl(void) {
 	gfarm_int32_t rv;
 
 	if (replica_check_ctrl_enabled()) {
-		/* info_time_start needs giant_lock() */
-		if (info_time_start != 0)
+		if (replica_check_is_running())
 			rv = GFM_PROTO_REPLICA_CHECK_STATUS_ENABLE_RUNNING;
 		else
 			rv = GFM_PROTO_REPLICA_CHECK_STATUS_ENABLE_STOPPED;
 	} else {
-		/* info_time_start needs giant_lock() */
+		/* report the status of replica_check_main() */
+		/* info_time_start is protected by giant_lock */
 		if (info_time_start != 0)
 			rv = GFM_PROTO_REPLICA_CHECK_STATUS_DISABLE_RUNNING;
 		else
@@ -1384,9 +1431,11 @@ replica_check_thread(void *arg)
 
 		replica_check_cond_wait();
 		replica_check_ctrl_wait(); /* if the ctrl is stop, wait here */
+		replica_check_is_running_update(1);
 		if (replica_check_main()) /* error occured, retry */
 			replica_check_cond_signal("retry_by_error",
 			    gfarm_metadb_heartbeat_interval);
+		replica_check_is_running_update(0);
 
 		/* call dead_file_copy_scan_deferred_all */
 		if (time(NULL) >= dfc_scan_time) {
@@ -1402,8 +1451,13 @@ replica_check_thread(void *arg)
 			    " next=%ld", (long)dfc_scan_time);
 		}
 		t = t - time(NULL);
-		if (t > 0)
-			gfarm_sleep(t);
+		while (t > 0) {
+			gfarm_sleep(1);
+			t -= 1;
+			/* use latest minimum_interval */
+			if (t > replica_check_minimum_interval_locked())
+				t = replica_check_minimum_interval_locked();
+		}
 	}
 	/*NOTREACHED*/
 	return (NULL);
