@@ -1,5 +1,5 @@
 /*
- * $Id: replica_check.h 9779 2015-06-30 06:31:35Z takuya-i $
+ * $Id: replica_check.h 10642 2017-10-26 11:21:31Z takuya-i $
  */
 
 void replica_check_init(void);
@@ -11,7 +11,9 @@ void replica_check_start_move(void);
 void replica_check_start_rep_request_failed(void);
 void replica_check_start_rep_result_failed(void);
 void replica_check_start_fsngroup_modify(void);
+void replica_check_start_host_is_not_busy(void);
 
 void replica_check_info(void);
 
 gfarm_error_t gfm_server_replica_check_ctrl(struct peer *, int, int);
+gfarm_error_t gfm_server_replica_check_status(struct peer *, int, int);
